"""
Sample API tests
"""
import pytest
import httpx
from pyneers.utils.request import (
    RestClient,
    AsyncGraphQLClient,
    AsyncRestClient,
    GraphQLClient,
    multiple_url_requests,
    hit_async_requests,
)

rest_api_base_url_sample_1 = "https://cat-fact.herokuapp.com/"
rest_api_base_url_sample_2 = "https://jsonplaceholder.typicode.com"


@pytest.mark.example
class TestApi:
    """
    Sample Test Class for reference
    """

    @pytest.fixture(scope="class", autouse=True)
    def setup_class(self, request, log, get_config):
        """
        setup method for test class

        Args:
            request: inbuilt pytest request
        """
        log.info("Setting up Class level stuffs\n")
        # pylint: disable=E1101
        request.cls.rest_base_url = get_config["url"]
        request.cls.rest_path_sample_1 = get_config["path"]
        # pylint: disable=E1101
        request.cls.rest_client = RestClient(base_url=self.rest_base_url)
        request.cls.graphql_base_url = "https://swapi-graphql.netlify.app/.netlify/functions/index"
        request.cls.graphql_client = GraphQLClient(self.graphql_base_url)

        yield

        log.info("Cleaning up Class level stuffs\n")

    @pytest.fixture(scope="function", autouse=True)
    def setup_method(self, log):
        """
        setup method for test functions
        """
        log.info("Setting up method level stuffs\n")
        yield
        log.info("Cleaning up method level stuffs\n")

    @pytest.mark.rest
    def test_rest_api(self, log):
        """
        Sample rest api test
        """
        log.info("Starting test for rest api")

        client = self.rest_client
        response = client.get(path=self.rest_path_sample_1)
        assert response.status_code == 200, f"Invalid Response:{response}"

        headers_rest_api_sample_2 = {"Content-Type": "application/json"}
        client = RestClient(rest_api_base_url_sample_2)
        response = client.post(
            path="/posts",
            json={"title": "New Post", "body": "Content"},
            headers=headers_rest_api_sample_2,
        )
        assert response.status_code == 201, f"Invalid Response:{response}"

        # Using session for http requests.
        client = RestClient(rest_api_base_url_sample_2, create_session=True)
        response = client.post(
            path="/posts",
            json={"title": "New Post", "body": "Content"},
            headers=headers_rest_api_sample_2,
        )
        client.close_session()
        assert response.status_code == 201, f"Invalid Response:{response}"

        # Using RestClient as context manager
        with RestClient(rest_api_base_url_sample_1) as client:
            path_restapi_sample_1 = "/facts"
            response = client.get(path=path_restapi_sample_1)
            assert response.status_code == 200, f"Invalid Response:{response}"

    @pytest.mark.graphql
    def test_graphql_api(self, log):
        """
        Sample graphql test
        """
        log.info("Starting test for graphql api")
        query = """
                query Query {
                allFilms {
                    films {
                    title
                    director
                    releaseDate
                    speciesConnection {
                        species {
                        name
                        classification
                        homeworld {
                            name
                        }
                        }
                    }
                    }
                }
                }
               """
        # pylint: disable=E1101
        response = self.graphql_client.execute_query(query)
        assert response.status_code == 200, f"Invalid Response:{response}"

        # Using session for http requests.
        client = GraphQLClient(self.graphql_base_url, create_session=True)
        response = client.execute_query(query)
        client.close_session()
        assert response.status_code == 200, f"Invalid Response:{response}"

        # Using GraphQLClient as context manager
        with GraphQLClient(self.graphql_base_url) as client:
            response = client.execute_query(query)
            assert response.status_code == 200, f"Invalid Response:{response}"

    def test_rest_api_multiple(self, log):
        """
        Sample multiple queries test
        """
        log.info("Starting test for multiple rest apis")
        requests_list_sample_multiple = [
            httpx.Request("GET", "https://jsonplaceholder.typicode.com/posts/1"),
            httpx.Request("GET", "https://reqres.in/api/users"),
        ]
        headers_sample_multiple = {"Authorization": "Bearer your_token", "User-Agent": "MyApp"}
        responses = multiple_url_requests(
            requests=requests_list_sample_multiple, headers=headers_sample_multiple
        )
        for response in responses:
            assert response.status_code == httpx.codes.OK, f"Invalid Response:{response}"


@pytest.mark.asyncio
class TestASyncApi:
    """
    Sample Test Class for reference
    """

    async def test_async_restapi(self, log):
        """
        Sample asynchronus rest api test
        """
        log.info("Starting test for asynchronus rest api")
        client = AsyncRestClient(rest_api_base_url_sample_1)
        path_restapi_sample_1 = "/facts"
        response = await client.get(path=path_restapi_sample_1)
        assert response.status_code == 200, f"Invalid Response:{response}"

        headers_restapi_sample_2 = {"Content-Type": "application/json"}
        client = AsyncRestClient(rest_api_base_url_sample_2)
        response = await client.post(
            path="/posts",
            payload={"title": "New Post", "body": "Content"},
            headers=headers_restapi_sample_2,
        )
        assert response.status_code == 201, f"Invalid Response:{response}"

        # Using AsyncRestClient as context manager
        async with AsyncRestClient(rest_api_base_url_sample_1) as client:
            path_restapi_sample_1 = "/facts"
            response = await client.get(path=path_restapi_sample_1)
            assert response.status_code == 200, f"Invalid Response:{response}"

    async def test_async_graphql(self, log):
        """
        Sample asynchronus GraphQL test
        """

        log.info("Starting test for asynchronus graphql queries")
        graphql_url = "https://swapi-graphql.netlify.app/.netlify/functions/index"
        query_graphql_sample = """
                query Query {
                allFilms {
                    films {
                    title
                    director
                    releaseDate
                    speciesConnection {
                        species {
                        name
                        classification
                        homeworld {
                            name
                        }
                        }
                    }
                    }
                }
                }
                """

        client = AsyncGraphQLClient(graphql_url)
        response = await client.execute_query(query=query_graphql_sample)
        assert response.status_code == httpx.codes.OK, f"Invalid Response:{response}"

        # Using AsyncGraphQLClient as context manager
        async with AsyncGraphQLClient(graphql_url) as client:
            response = await client.execute_query(query=query_graphql_sample)
            assert response.status_code == 200, f"Invalid Response:{response}"

    async def test_api_multiples(self, log):
        """
        Sample multiple asynchronus queries test
        """
        log.info("Starting test for multiple api calls")
        requests_list_sample_multiple = [
            httpx.Request("GET", "https://jsonplaceholder.typicode.com/posts/1"),
            httpx.Request("GET", "https://reqres.in/api/users"),
        ]
        headers_sample_multiple = {"Authorization": "Bearer your_token", "User-Agent": "MyApp"}

        responses = await hit_async_requests(
            requests=requests_list_sample_multiple, headers=headers_sample_multiple
        )
        for response in responses:
            assert response.status_code == httpx.codes.OK, f"Invalid Response:{response}"
