"""
This module contains test cases for interacting with a real MongoDB database using DBUtils.
"""
import pytest
import pandas as pd
from pyneers.utils.db import DBUtils, DBConfig
from pyneers.utils.db.db_operations_handler import MongoQueryObject


# pylint: disable=E1129
# pylint: disable=duplicate-code
@pytest.mark.skip("Skipping as the test need to run with local db")
@pytest.mark.realmongo
class TestRealDb:
    """
    This class contains test cases for interacting with a real MongoDB database using DBUtils.
    """

    @pytest.fixture(scope="class", autouse=True)
    def setup(self, request):
        """
        Setup method for configuring the test environment with a database configuration.
        """
        db_config = DBConfig(
            username="",
            password="",
            host="127.0.0.1",
            port="27017",
            database_name="my_db",
            db_type="mongodb",
            auth_source="admin",
            auth_mechanism="SCRAM-SHA-256",
        )

        request.cls.config = db_config

    @pytest.mark.realmongocontextfind
    def test_context_find(self):
        """
        Test case for finding data from the database using DBUtils and verifying the results.
        """
        with DBUtils(self.config) as db_utils:
            data = {"name": "Bob", "age": 25}
            query_object_insert = MongoQueryObject(
                collection_name="students", data=data, operation="insert"
            )
            db_utils.execute_query(query_object_insert)

            query = {"name": "Bob"}
            query_object_find = MongoQueryObject(
                collection_name="students", query=query, operation="find"
            )

            result = db_utils.execute_query(query_object_find)

            assert result is not None
            result_list = list(result)
            found_data = result_list[0]
            assert found_data["name"] == "Bob"
            assert found_data["age"] == 25

    @pytest.mark.realmongocontextfindpandas
    def test_context_find_pandas(self):
        """
        Test case for finding data from the database using DBUtils and verifying the results.
        """
        with DBUtils(self.config) as db_utils:
            data = {"name": "Bob", "age": 25}
            query_object_insert = MongoQueryObject(
                collection_name="students", data=data, operation="insert"
            )
            db_utils.execute_query(query_object_insert)

            query = {"name": "Bob"}
            query_object_find = MongoQueryObject(
                collection_name="students", query=query, operation="find"
            )

            result = db_utils.execute_query(query_object_find, read_pandas=True)

            assert isinstance(result, pd.DataFrame)

    @pytest.mark.realmongocontextupdate
    def test_context_update(self):
        """
        Test case for updating data from the database using DBUtils and verifying the results.
        """
        with DBUtils(self.config) as db_utils:
            # First, insert a record for testing
            initial_data = {"name": "Charlie", "age": 28}
            query_object_insert = MongoQueryObject(
                collection_name="testcollection", data=initial_data, operation="insert"
            )
            db_utils.execute_query(query_object_insert)

            update_data = {"age": 29}
            query = {"name": "Charlie", "age": 28}
            query_object_update = MongoQueryObject(
                collection_name="testcollection", query=query, data=update_data, operation="update"
            )

            result = db_utils.execute_query(query_object_update)

            assert result is not None
            assert result.modified_count > 0

            query_after_update = {"name": "Charlie", "age": 29}
            query_object_find_after_update = MongoQueryObject(
                collection_name="testcollection", query=query_after_update, operation="find"
            )

            updated_result = db_utils.execute_query(query_object_find_after_update)

            assert updated_result is not None
            updated_data_found = list(updated_result)
            assert updated_data_found[0]["name"] == initial_data["name"]
            assert updated_data_found[0]["age"] == 29

    @pytest.mark.realmongocontextdelete
    def test_context_delete(self):
        """
        Test case for deleting data from the database using DBUtils and verifying the results.
        """
        with DBUtils(self.config) as db_utils:
            data = {"name": "David", "age": 32}
            query_object_insert = MongoQueryObject(
                collection_name="students", data=data, operation="insert"
            )
            db_utils.execute_query(query_object_insert)

            query = {"name": "David"}
            query_object_delete = MongoQueryObject(
                collection_name="students", query=query, operation="delete"
            )

            result = db_utils.execute_query(query_object_delete)

            assert result is not None
            assert result.deleted_count > 0
