"""
This module contains test cases for interacting with a real SQL database using DBUtils.
"""

import pytest
import pandas as pd
from pyneers.utils.db import DBUtils, DBConfig


# pylint: disable=E1129
@pytest.mark.skip("Skipping as the test need to run with local db")
@pytest.mark.realsqldb
class TestRealDb:
    """
    This class contains test cases for interacting with a real SQL database using DBUtils.
    """

    @pytest.fixture(scope="class", autouse=True)
    def setup(self, request):
        """
        Setup method for configuring the test environment with a database configuration.
        """

        db_config = DBConfig(
            username="",
            password="",
            host="127.0.0.1",
            port="5432",
            database_name="my_pgdb",
            db_type="postgresql",
        )
        request.cls.config = db_config

    @pytest.mark.realdbcontextselect
    def test_context_select(self, log):
        """
        Test case for selecting data from the database using DBUtils and verifying the results.
        """

        with DBUtils(self.config) as db_utils:
            query = "Select * from employees"

            res = db_utils.execute_query(query)
            log.info(res)
            for each in res:
                assert len(each) == 3
            assert res is not None

    @pytest.mark.realdbcontextselectpandas
    def test_context_select_pandas(self, log):
        """
        Test case for selecting data from the database using DBUtils and verifying the results.
        """

        with DBUtils(self.config) as db_utils:
            query = "Select * from employees"

            res = db_utils.execute_query(query, read_pandas=True)
            log.info(res)
            assert isinstance(res, pd.DataFrame)

    @pytest.mark.realdbcontextinsert
    def test_context_insert(self, log):
        """
        Test case for inserting data from the database using DBUtils and verifying the results.
        """

        with DBUtils(self.config) as db_utils:
            query = "INSERT INTO employees (id, first_name, last_name) VALUES (34, 'John', 'Doe');"
            res = db_utils.execute_query(query)
            log.info(res)
            assert res.rowcount != 0

            query = "Select * from employees where id = 34;"
            res = db_utils.execute_query(query)
            log.info(res)
            assert res == [(34, "John", "Doe")]

    @pytest.mark.realdbcontextinsertmany
    def test_context_insert_many(self, log):
        """
        Test case for inserting data from the database using DBUtils and verifying the results.
        """

        with DBUtils(self.config) as db_utils:
            data = [
                {"id": 1, "first_name": "Alice", "last_name": "Smith"},
                {"id": 2, "first_name": "Bob", "last_name": "Johnson"},
                {"id": 3, "first_name": "Charlie", "last_name": "Brown"},
            ]

            query = "INSERT INTO employees (id, first_name, last_name) VALUES (:id, :first_name, :last_name)"
            res = db_utils.execute_query(query, params=data)
            log.info(res)
            assert res.rowcount == len(data)

    @pytest.mark.realdbcontextupdate
    def test_context_update(self, log):
        """
        Test case for updating data from the database using DBUtils and verifying the results.
        """
        with DBUtils(self.config) as db_utils:
            # First, insert a record for testing
            insert_query = (
                "INSERT INTO employees (id, first_name, last_name) VALUES (43, 'Jane', 'Smith');"
            )
            db_utils.execute_query(insert_query)

            # Now, update the inserted record
            update_query = "UPDATE employees SET last_name = 'Johnson' WHERE id = 43;"
            res = db_utils.execute_query(update_query)
            log.info(res)
            assert res.rowcount != 0

            # Verify the record has been updated
            select_query = "SELECT * FROM employees WHERE id = 43;"
            res = db_utils.execute_query(select_query)
            log.info(res)
            assert res == [(43, "Jane", "Johnson")]

    @pytest.mark.realdbcontextupdatemany
    def test_context_update_many(self, log):
        """
        Test case for updating data in the database using DBUtils and verifying the results.
        """

        with DBUtils(self.config) as db_utils:
            data = [
                {"id": 1, "first_name": "New_Alice", "last_name": "New_Smith"},
                {"id": 2, "first_name": "New_Bob", "last_name": "New_Johnson"},
                {"id": 3, "first_name": "New_Charlie", "last_name": "New_Brown"},
            ]

            query = """
                UPDATE employees
                SET first_name = :first_name, last_name = :last_name
                WHERE id = :id
            """
            res = db_utils.execute_query(query, params=data)
            log.info(res)

            # Verify the number of rows updated
            assert res.rowcount == len(data)

    @pytest.mark.realdbcontextdelete
    def test_context_delete(self, log):
        """
        Test case for deleting data from the database using DBUtils and verifying the results.
        """
        with DBUtils(self.config) as db_utils:
            # First, insert a record for testing
            insert_query = (
                "INSERT INTO employees (id, first_name, last_name) VALUES (44, 'Mark', 'Davis');"
            )
            db_utils.execute_query(insert_query)

            # Now, delete the inserted record
            delete_query = "DELETE FROM employees WHERE id = 44;"
            res = db_utils.execute_query(delete_query)
            log.info(res)
            assert res.rowcount != 0

            # Verify that the record has been deleted
            select_query = "SELECT * FROM employees WHERE id = 44;"
            res = db_utils.execute_query(select_query)
            log.info(res)
            assert len(res) == 0

    @pytest.mark.realdbcontextdeletemany
    def test_context_delete_many(self, log):
        """
        Test case for deleting data from the database using DBUtils and verifying the results.
        """

        with DBUtils(self.config) as db_utils:
            data = [{"id": 1}, {"id": 2}, {"id": 3}]  # List of IDs to delete

            query = "DELETE FROM employees WHERE id = :id"
            res = db_utils.execute_query(query, params=data)
            log.info(res)

            # Verify the number of rows deleted
            assert res.rowcount == len(data)
