"""
Contains all playwright related test fixtures
"""
from contextlib import suppress
from pathlib import Path
from typing import Any, Callable, Dict, Generator, List, Optional

import pytest
from playwright.sync_api import (
    Browser,
    BrowserContext,
    BrowserType,
    Error,
    Page,
    Playwright,
    sync_playwright,
)
from slugify import slugify

from pyneers.utils.logger import CustomLogger


@pytest.fixture(scope="class", autouse=True)
def log():
    """
    Test Logger
    """
    logger = CustomLogger.getLogger("pytest")
    return logger


@pytest.fixture(autouse=True, scope="function")
def test_listeners(request, log):
    """
    Adds additional info log regarding execution status
    request.node is an "item" because we use the default
    "function" scope
    Args:
        request (fixture):
        log (fixture):
    """
    yield
    if request.node.rep_call.failed:
        log.info(f"Test failed!!: {request.node.rep_call.longrepr.reprcrash.message}")
    elif request.node.rep_call.passed:
        log.info(f"Test passed: {request.node.nodeid}")
    elif request.node.rep_call.skipped:
        log.info(f"Test skipped: {request.node.nodeid}")


@pytest.hookimpl(tryfirst=True, hookwrapper=True)
def pytest_runtest_makereport(item: Any) -> Generator[None, Any, None]:
    """
    Making test result information available in fixtures
    https://docs.pytest.org/en/latest/example/simple.html#making-test-result-information-available-in-fixtures
    Execute all other hooks to obtain the report object
    Args:
        item (Any):
    Yields:
        Generator[None, Any, None]:
    """

    outcome = yield
    rep = outcome.get_result()

    test_fn = item.obj
    rep.extra = []
    # get the test level docstrings as test long descriptions
    docstring = getattr(test_fn, "__doc__")
    if docstring:
        rep.description = docstring
    # set a report attribute for each phase of a call, which can
    # be "setup", "call", "teardown"

    if rep.when == "setup":
        rep.extra.append(
            {
                "name": "Browser Channel",
                "format": "text",
                "content": f"{item.funcargs.get('browser_channel', '')}",
            }
        )
    if rep.when == "call" and rep.failed is True:
        rep.extra.append(
            {
                "name": "Screenshot",
                "format": "image",
                "content": str(Path.cwd() / "reports" / slugify(rep.nodeid) / f"{item.name}.png"),
            }
        )
        rep.extra.append(
            {
                "name": "Recording",
                "format": "video",
                "content": str(Path.cwd() / "reports" / slugify(rep.nodeid) / f"{item.name}.webm"),
            },
        )

    setattr(item, "rep_" + rep.when, rep)


@pytest.fixture(scope="class")
def browser_type_launch_args(pytestconfig: Any) -> Dict:
    """
    Prepares browser launch args
    Args:
        pytestconfig (Any):
    Returns:
        Dict:
    """
    launch_options = {}
    headed_option = pytestconfig.getoption("--headed")
    if headed_option:
        launch_options["headless"] = False
    browser_channel_option = pytestconfig.getoption("--browser-channel")
    if browser_channel_option:
        launch_options["channel"] = browser_channel_option
    slowmo_option = pytestconfig.getoption("--slowmo")
    if slowmo_option:
        launch_options["slow_mo"] = slowmo_option
    return launch_options


def _build_artifact_test_folder(
    pytestconfig: Any, request: pytest.FixtureRequest, folder_or_file_name: str
) -> str:
    """
    Prepares an artifacts folder for storing test results

    Args:
        pytestconfig (Any):
        request (pytest.FixtureRequest):
        folder_or_file_name (str):

    Returns:
        str: _description_
    """
    output_dir = pytestconfig.getoption("--output")
    return str(Path(output_dir) / slugify(request.node.nodeid) / folder_or_file_name)


@pytest.fixture(scope="class")
def browser_context_args(pytestconfig: Any, playwright: Playwright, device: Optional[str]) -> Dict:
    """
    Prepares Playwright browser context args

    Args:
        pytestconfig (Any):
        playwright (Playwright):
        device (Optional[str]):

    Returns:
        Dict:
    """
    context_args = {}
    if device:
        context_args.update(playwright.devices[device])
    base_url = pytestconfig.getoption("--base-url")
    if base_url:
        context_args["base_url"] = base_url

    video_option = pytestconfig.getoption("--video")
    capture_video = video_option in ("on", "retain-on-failure")
    if capture_video:
        context_args["record_video_dir"] = "artifacts/playwright"

    return context_args


@pytest.fixture(scope="class")
def playwright() -> Generator[Playwright, None, None]:
    """
    Starts playwright in sync mode

    Yields:
        Generator[Playwright, None, None]:
    """
    pw = sync_playwright().start()
    yield pw
    pw.stop()


@pytest.fixture(scope="class")
def browser_type(playwright: Playwright, browser_name: str) -> BrowserType:
    """
    Get browser type

    Args:
        playwright (Playwright):
        browser_name (str):

    Returns:
        BrowserType:
    """
    return getattr(playwright, browser_name)


@pytest.fixture(scope="class")
def launch_browser(
    browser_type_launch_args: Dict,
    browser_type: BrowserType,
) -> Callable[..., Browser]:
    """
    Launches playwright broswer instance

    Args:
        browser_type_launch_args (Dict):
        browser_type (BrowserType):

    Returns:
        Callable[..., Browser]:
    """

    def launch(**kwargs: Dict) -> Browser:
        launch_options = browser_type_launch_args | kwargs
        browser = browser_type.launch(**launch_options)
        return browser

    return launch


@pytest.fixture(scope="class")
def browser(launch_browser: Callable[[], Browser]) -> Generator[Browser, None, None]:
    """
    Playwright Browser

    Args:
        launch_browser (Callable[[], Browser]):

    Yields:
        Generator[Browser, None, None]:
    """
    browser = launch_browser()
    yield browser
    browser.close()


@pytest.fixture(scope="class")
def context(
    browser: Browser,
    browser_context_args: Dict,
    pytestconfig: Any,
    request: pytest.FixtureRequest,
) -> Generator[BrowserContext, None, None]:
    """
    Playwright Browser context

    Args:
        browser (Browser):
        browser_context_args (Dict):
        pytestconfig (Any):
        request (pytest.FixtureRequest):

    Yields:
        Generator[BrowserContext, None, None]:
    """

    def handle_new_page(page: Page):
        pages.append(page)

    pages: List[Page] = []
    context = browser.new_context(**browser_context_args)
    context.on("page", handle_new_page)

    tracing_option = pytestconfig.getoption("--tracing")
    capture_trace = tracing_option in ("on", "retain-on-failure")
    if capture_trace:
        context.tracing.start(
            name=slugify(request.node.nodeid),
            screenshots=True,
            snapshots=True,
        )

    yield context

    if capture_trace:
        retain_trace = tracing_option == "on" or (
            request.node.rep_call.failed and tracing_option == "retain-on-failure"
        )
        if retain_trace:
            trace_path = _build_artifact_test_folder(pytestconfig, request, "trace.zip")
            context.tracing.stop(path=trace_path)
        else:
            context.tracing.stop()

    screenshot_option = pytestconfig.getoption("--screenshot")
    capture_screenshot = screenshot_option == "on"
    # rep_call for screenshot disabled
    # or (
    #    request.node.rep_call.failed and screenshot_option == "only-on-failure"
    # )
    if capture_screenshot:
        for page in pages:
            screenshot_path = _build_artifact_test_folder(
                pytestconfig, request, f"{request.node.name}.png"
            )
            with suppress(Error):
                page.screenshot(timeout=5000, path=screenshot_path)

    context.close()

    video_option = pytestconfig.getoption("--video")
    preserve_video = video_option == "on" or (
        video_option == "retain-on-failure" and request.node.rep_call.failed
    )
    if preserve_video:
        for page in pages:
            video = page.video
            if not video:
                continue
            with suppress(Error):
                file_name = request.node.name + ".webm"
                video.save_as(path=_build_artifact_test_folder(pytestconfig, request, file_name))


@pytest.fixture(scope="class")
def page(context: BrowserContext) -> Generator[Page, None, None]:
    """
    Playwright page

    Args:
        context (BrowserContext):

    Yields:
        Generator[Page, None, None]:
    """
    page = context.new_page()
    yield page


@pytest.fixture(scope="class")
def browser_name(pytestconfig: Any) -> Optional[str]:
    """
    Returns browser name list

    Args:
        pytestconfig (Any):

    Returns:
        Optional[str]:
    """
    browser_names = pytestconfig.getoption("--browser")
    if len(browser_names) == 0:
        return "chromium"
    if len(browser_names) == 1:
        return browser_names[0]
    return browser_names[0]


@pytest.fixture(scope="class", autouse=True)
def browser_channel(pytestconfig: Any) -> Optional[str]:
    """
    Broswer channel value passed in cli

    Args:
        pytestconfig (Any):

    Returns:
        Optional[str]:
    """
    return pytestconfig.getoption("--browser-channel")


@pytest.fixture(scope="class")
def device(pytestconfig: Any) -> Optional[str]:
    """
    Device value passed in cli

    Args:
        pytestconfig (Any):

    Returns:
        Optional[str]:
    """
    return pytestconfig.getoption("--device")
