""" ArgoCD API pydnatic models """
from typing import Any, List, Mapping, Optional
from pydantic import BaseModel


class JWTRequest(BaseModel):
    """JWT request model"""

    password: str
    username: str


class JWTResponse(BaseModel):
    """JWT response model"""

    token: str


class DefaultError(BaseModel):
    """Default error model"""

    code: int
    details: list[Mapping[str, str]] | None
    error: str
    message: str


class _ResourcesInfoItem(BaseModel):
    """Resources info item model"""

    capacity: str
    requestedByApp: str
    requestedByNeighbors: str
    resourceName: str


class _SystemInfo(BaseModel):
    """System info model"""

    architecture: str
    bootID: str
    containerRuntimeVersion: str
    kernelVersion: str
    kubeProxyVersion: str
    kubeletVersion: str
    machineID: str
    operatingSystem: str
    osImage: str
    systemUUID: str


class _Host(BaseModel):
    """Host model"""

    name: str
    resourcesInfo: List[_ResourcesInfoItem]
    systemInfo: _SystemInfo


class _Health(BaseModel):
    """Health model"""

    message: Optional[str] = None
    status: str


class _InfoItem(BaseModel):
    """Info item model"""

    name: str
    value: Optional[str] = None


class _Port(BaseModel):
    """Port model"""

    error: str
    port: int
    protocol: str


class _Ingress(BaseModel):
    """Ingress model"""

    hostname: Optional[str] = None
    ip: Optional[str] = None
    ports: Optional[List[_Port]] = None


class _TargetRef(BaseModel):
    """Target ref model"""

    group: Optional[str] = None
    kind: str
    name: str
    namespace: str
    uid: Optional[str] = None
    version: Optional[str] = None


class _NetworkingInfo(BaseModel):
    """Networking info model"""

    externalURLs: Optional[List[str]] = None
    ingress: Optional[List[_Ingress]] = None
    labels: Optional[Mapping[str, str]] = None
    targetLabels: Optional[Mapping[str, str]] = None
    targetRefs: Optional[List[_TargetRef]] = None


class _ParentRef(BaseModel):
    """Parent ref model"""

    group: Optional[str] = None
    kind: str
    name: str
    namespace: str
    uid: str
    version: Optional[str] = None


class _Node(BaseModel):
    """Node model"""

    createdAt: str
    health: Optional[_Health] = None
    images: Optional[List[str]] = None
    info: Optional[List[_InfoItem]] = None
    networkingInfo: Optional[_NetworkingInfo] = None
    parentRefs: Optional[List[_ParentRef]] = None
    group: Optional[str] = None
    uid: str
    name: str
    kind: str
    version: Optional[str] = None
    namespace: str
    resourceVersion: str


class ResourceTree(BaseModel):
    """Resource tree model"""

    hosts: Optional[List[_Host]] = None
    nodes: List[_Node]
    orphanedNodes: List[_Node]


class _Metadata(BaseModel):
    """Metadata model"""

    name: str
    namespace: str
    uid: str
    resourceVersion: str
    generation: int
    creationTimestamp: str
    ownerReferences: Optional[Any] = None
    finalizers: List[str]
    managedFields: Any


class _FileParameter(BaseModel):
    """File parameter model"""

    name: str
    path: str


class _Helm(BaseModel):
    """Helm model"""

    valueFiles: Optional[List[str]] = None
    fileParameters: Optional[List[_FileParameter]] = None


class _Source(BaseModel):
    """Source model"""

    repoURL: Optional[str] = None
    path: Optional[str] = None
    targetRevision: Optional[str] = None
    helm: Optional[_Helm] = None


class _Destination(BaseModel):
    """Destination model"""

    server: str
    namespace: str


class _Automated(BaseModel):
    """Automated model"""

    prune: Optional[bool] = None
    selfHeal: Optional[bool] = None


class _SyncPolicy(BaseModel):
    """Sync policy model"""

    automated: Optional[_Automated] = None
    syncOptions: Optional[List[str]] = None


class _Spec(BaseModel):
    """Spec model"""

    source: _Source
    destination: _Destination
    project: str
    syncPolicy: Optional[_SyncPolicy] = None


class _Resource(BaseModel):
    """Resource model"""

    version: str
    kind: str
    namespace: Optional[str] = None
    name: str
    hook: Optional[bool] = None
    requiresPruning: Optional[bool] = None
    status: Optional[str] = None
    health: Optional[_Health] = None
    group: Optional[str] = None


class _ComparedTo(BaseModel):
    """Compared to model"""

    source: _Source
    destination: _Destination


class _Sync(BaseModel):
    """Sync model"""

    status: Optional[str] = None
    comparedTo: Optional[_ComparedTo] = None
    revision: Optional[str] = None


class _HistoryItem(BaseModel):
    """History item model"""

    revision: str
    deployedAt: str
    id: int
    source: _Source
    deployStartedAt: str


class _Condition(BaseModel):
    """Condition model"""

    type: str
    message: str
    lastTransitionTime: str


class _InitiatedBy(BaseModel):
    """Initiated by model"""

    automated: Optional[bool] = None


class _Retry(BaseModel):
    """Retry model"""

    limit: Optional[int] = None


class _Operation(BaseModel):
    """Operation model"""

    sync: Optional[_Sync] = None
    initiatedBy: Optional[_InitiatedBy] = None
    retry: Optional[_Retry] = None


class _SyncResult(BaseModel):
    """Sync result model"""

    resources: List[_Resource]
    revision: str
    source: _Source


class _OperationState(BaseModel):
    """Operation state model"""

    operation: _Operation
    phase: str
    message: str
    syncResult: _SyncResult
    startedAt: str
    finishedAt: str


class _Summary(BaseModel):
    """Summary model"""

    externalURLs: Optional[List[str]] = None
    images: Optional[List[str]] = None


class _Status(BaseModel):
    """Status model"""

    resources: Optional[List[_Resource]] | None
    sync: _Sync
    health: _Health
    history: List[_HistoryItem]
    conditions: Optional[List[_Condition]] = None
    reconciledAt: str
    operationState: _OperationState
    sourceType: Optional[str] = None
    summary: Optional[_Summary] = None


class Application(BaseModel):
    """Application model"""

    metadata: _Metadata
    spec: _Spec
    status: _Status


class DegradedResource(BaseModel):
    """Degraded resource model"""

    name: str
    sync: str
    health: str
    message: Optional[str] = None
