"""
    Utility class for creating files and directories during initialization.
"""
from pathlib import Path
from colorama import Fore, Style


dockerfile_content = """
# Use a base image that supports Python
FROM --platform=linux/amd64 python:3.11.4-slim

# Set the working directory inside the container
WORKDIR /app

# Copy the test file to the container
COPY . .
COPY entrypoint.sh /entrypoint.sh
RUN chmod +x /entrypoint.sh

# Install poetry
RUN pip install poetry

# Install project dependencies using poetry
RUN poetry install

# Set PYTHONPATH to current workdir
ENV PYTHONPATH="${PYTHONPATH}:/app"

# Set the entrypoint to the script
ENTRYPOINT ["/entrypoint.sh"]
"""

conftest_content = """
# pylint: disable=W0401,W0614
from pyneers.fixtures.playwright import *  # noqa F401

# pylint: disable=W0401,W0614
from pyneers.fixtures.log import *  # noqa F401

#sample for rest api testing
from jinja2 import Template
import pytest
import yaml
import os.path

current_script_path = os.path.abspath(__file__)
current_script_directory = os.path.dirname(current_script_path)
artifacts_path = os.path.join(current_script_directory,"artifacts")


def pytest_configure(config):
    report_path = os.path.join(artifacts_path, "report.html")
    config.option.htmlpath = report_path

@pytest.fixture(scope="class")
def get_config():
    file_path=os.path.join(current_script_directory,"configs/config.j2")
    with open(file_path, 'r') as template_file:
        template_content = template_file.read()
        # Create a Jinja2 Template object
        template = Template(template_content)

    rest_api_data ={
        'url_1': "https://cat-fact.herokuapp.com/",
        'path_1': "/facts",
    }

    rendered_template = template.render(**rest_api_data)

    parsed_dict = yaml.safe_load(rendered_template)

    return parsed_dict["rest_api"]["sample_1"]

"""

pytestini_content = """
[pytest]
# noqa F501
addopts =
    -v -s --html=artifacts/report.html --self-contained-html
    --alluredir=artifacts/allure-results --capture sys --output=artifacts/playwright
    --cov=pyneers --cov-report=term --cov-report=html:artifacts/coverage_report
junit_family = xunit2

log_cli = 1
log_cli_level = INFO
log_cli_format = %(asctime)s [%(levelname)8s] %(message)s (%(filename)s:%(lineno)s)
log_cli_date_format=%Y-%m-%d %H:%M:%S

log_file = artifacts/pytest.log
log_file_level = DEBUG
log_file_format = %(asctime)s [%(levelname)8s] %(message)s (%(filename)s:%(lineno)s)
log_file_date_format=%Y-%m-%d %H:%M:%S

markers =
    example: example tests.
    rest: rest api.
    graphql: graphql api.
    ui: playwright tests.

"""

pre_commit_content = """
repos:
  - repo: local
    hooks:
      - id: toml-validation
        name: Validate TOML files
        entry: tomlcheck
        language: system
        files: '\.toml$' # noqa F605

  - repo: https://github.com/psf/black
    rev: 23.3.0
    hooks:
      - id: black
        args: [--line-length=100]

  - repo: https://github.com/PyCQA/flake8
    rev: 6.0.0
    hooks:
      - id: flake8

  - repo: local
    hooks:
    -   id: pylint
        name: pylint
        entry: pylint
        language: system
        types: [python]
        require_serial: true
        args:
          - "--rcfile=pyproject.toml"

  - repo: https://github.com/pre-commit/pre-commit-hooks
    rev: v4.4.0
    hooks:
      - id: check-case-conflict
      - id: check-docstring-first
      - id: check-json
      - id: check-yaml
        exclude: ^deployment/test-helm-chart/templates/pod\.yaml$ # noqa F605
      - id: check-merge-conflict
      - id: check-symlinks
      - id: check-vcs-permalinks
      - id: debug-statements
      - id: end-of-file-fixer
      - id: trailing-whitespace

  - repo: https://github.com/Yelp/detect-secrets
    rev: v1.4.0
    hooks:
      - id: detect-secrets

"""

workflow_content = """
name: Build and Lint

on:
  pull_request:
    branches:
      - "*"

jobs:
  build:
    runs-on: ubuntu-latest
    steps:
      #----------------------------------------------
      #       check-out repo and set-up python
      #----------------------------------------------
      - name: Checkout code
        uses: actions/checkout@v3
      - name: Set up Python
        uses: actions/setup-python@v4
        with:
          python-version: 3.11.4
      #----------------------------------------------
      #  -----  install & configure poetry  -----
      #----------------------------------------------
      - name: Install Poetry
        uses: snok/install-poetry@v1
        with:
          virtualenvs-create: true
          virtualenvs-in-project: true
          installer-parallel: true
        env:
          GITHUB_TOKEN: ${{ secrets.GITHUB_TOKEN }}
      #----------------------------------------------
      #       load cached venv if cache exists
      #----------------------------------------------
      - name: Load cached venv
        id: cached-poetry-dependencies
        uses: actions/cache@v3
        with:
          path: .venv
          key: venv-${{ runner.os }}-${{ steps.setup-python.outputs.python-version }}-${{ hashFiles('**/poetry.lock') }}
      #----------------------------------------------
      # install dependencies if cache does not exist
      #----------------------------------------------
      - name: Install dependencies
        if: steps.cached-poetry-dependencies.outputs.cache-hit != 'true'
        run: poetry install --no-interaction --no-root
      #----------------------------------------------
      #              run Black Formatting
      #----------------------------------------------
      - name: Black
        uses: microsoft/action-python@0.7.0
        with:
          black: true
          workdir: .
      #----------------------------------------------
      #              run Pylint
      #----------------------------------------------
      # - name: Pylint
      #   uses: microsoft/action-python@0.7.0
      #   with:
      #     pylint: true
      #     workdir: .
      #     INPUT_PYLINT_RC: .pylintrc
      #----------------------------------------------
      #              run linting using Flake8
      #----------------------------------------------
      - name: Flake8
        uses: microsoft/action-python@0.7.0
        with:
          flake8: true
          workdir: .
      #----------------------------------------------
      #              run test suite
      #----------------------------------------------
      - name: Run tests
        run: |
          source .venv/bin/activate
          export PYTHONPATH=$GITHUB_WORKSPACE
          pytest --collect-only -qq

"""

readme_content = """
Welcome to Pyneers!


### Running Pyneers Tests
After initializing your Pyneers project, you can use `Pytest` to run your tests.

Pyneers is built on top of Pytest, so you can use all the standard Pytest commands to run your tests.
```sh
pytest <path_to_test_file_or_folder>
```
Replace <path_to_test_file_or_folder> with the path to the test file or folder you want to run.

---
### Docs

* [pyneers](https://effective-adventure-29wlj4q.pages.github.io/)
* [playwright](https://playwright.dev/docs/intro)
* [requests](https://requests.readthedocs.io/en/latest/)
* [httpx](https://www.python-httpx.org/)



"""

configsj2_content = """
#sample for rest api testing
rest_api:
    sample_1:
        url: {{ url_1 }}
        path: {{ path_1 }}


"""

entrypoint_content = """
#!/bin/bash

# Activate the virtual environment
source .venv/bin/activate

# Print the value of VIRTUAL_ENV
echo "VIRTUAL_ENV: $VIRTUAL_ENV"

# Print the command passed as arguments
echo "Received arguments: $@"

# Split the arguments into an array
IFS=' ' read -ra ARGS <<< "$@"

# Get the first argument (command)
CMD="${ARGS[0]}"
echo "First argument: $CMD"

# Check if "pytest" is present as the 1st argument
if [[ "$CMD" == "pytest" ]]; then
    # Run pytest with the provided arguments
    echo "executing pytest command: pytest ${ARGS[@]:1}"
    echo "############################## Test Execution Start ##############################"
    pytest "${ARGS[@]:1}"
    echo "############################## Test Execution End ################################"
else
    # Execute the command provided as arguments
    echo "executing command: ${ARGS[@]:0}"
    "${ARGS[@]:0}"
    echo "Custom command executed"
fi

"""


class FileDirUtility:
    """
    A utility class for creating directories and files.

    """

    def __init__(self, project_name):
        """
        Initialize a new instance of FileDirectoryUtility.

        Args:
            project_name (str): The name of the project.
        """
        self.project_name = project_name

    def directory_maker(self, directory_name, log_info=True) -> str:
        """
        Create a directory.

        Args:
            directory (str): The name of the directory to create.
            log_info (bool, optional): Whether to log information about the creation. Default is True.

        Returns:
            str: The path of the created directory.
        """
        directory_path = Path(self.project_name) / directory_name
        if log_info:
            print(f"\nCreating folder:{Fore.LIGHTGREEN_EX}{directory_name}{Style.RESET_ALL}!\n")
        directory_path.mkdir(exist_ok=True)
        return str(directory_path)

    def file_maker(self, file_name, content, path="", log_info=True) -> str:
        """
        Create a file with the given content.

        Args:
            file_name (str): The name of the file to create.
            content (str): The content to write to the file.
            path(str): The path to the file relative to present directory.
            log_info (bool, optional): Whether to log information about the creation. Default is True.

        Returns:
            str: The path of the created file.
        """
        if log_info:
            print(f"\nCreating file:{Fore.LIGHTYELLOW_EX}{file_name}{Style.RESET_ALL}!\n")
        file_path = Path(self.project_name) / path / file_name
        with open(file_path, "w", encoding="utf-8") as file:
            file.write(content)
        return str(file_path)
