"""Base class for file handlers"""
from pathlib import Path
from abc import ABC
from pyneers.utils.logger import CustomLogger
from pyneers.utils.exceptions import FilePathError, FileSizeError, FileTypeError

LOG = CustomLogger.getLogger(__name__)
SIZE_LIMIT = 500  # in MB
TYPES = ["txt", "yaml", "yml", "json", "csv", "xlsx"]


class BaseHandler(ABC):
    """Base class for file handlers of different types"""

    def __init__(self, file_path: str):
        """
        Initialize the BaseHandler with file path

        Args:
            file_path (str): Path of the file on which operations to be performed
        """
        self.file_path = Path(file_path)
        self.file = None
        self.file_type = None

    def validate_file(self):
        """
        Description:
            Responsible for performing following file validations:
            - File Path: file path must be correct and should exist on the system
            - File Size: file size must be less than 500 MB
            - File Type: file type should be either json, yaml, txt, or csv.

        """
        if not self.file_path.is_file():
            LOG.error("File path is not correct!")
            raise FilePathError()

        file_stats = self.file_path.stat()
        file_size = file_stats.st_size / (1024 * 1024)
        if file_size > SIZE_LIMIT:
            LOG.error(f"File exceeding the size limit of {SIZE_LIMIT} MB!")
            raise FileSizeError()

        self.file_type = self.file_path.suffix[1:].lower()
        if self.file_type not in TYPES:
            LOG.error(f"File type not supported. Supported file formats are {TYPES}")
            raise FileTypeError()
        return True

    def read(self):
        """read method for file handlers"""

    def cache_read(self):
        """read method for file handlers using cache"""

    def write(self, data):
        """write method for file handlers"""

    def append(self, data):
        """append method for file handlers"""
