"""
Bash utils module
"""

import subprocess
import concurrent.futures
from typing import Union, List, Dict, Tuple, Optional
from pathlib import Path

from pyneers.utils.logger import CustomLogger

LOG = CustomLogger.getLogger(__name__)


class BashHelper:
    """
    bash helper class for running bash commands in python
    """

    def __init__(self):
        """
        PASS:
        """

    def execute_command(self, command: str) -> str:
        """
         this method take shell command as string and return string as output after
         success else return error msg for command failed


        Args:
            command (str): command passed for execution

        Returns:
            str: command output either stdout or stderr
        """
        try:
            LOG.info(f"command execution started : {command}")
            result = subprocess.check_output(
                command, shell=True, universal_newlines=True, stderr=subprocess.STDOUT
            )
            LOG.debug(f"command executed succesfully : {command}")
            result = result.strip()
            return result
        except subprocess.CalledProcessError as e:
            error_msg = f"Command execution failed with error: {e}"
            LOG.error(error_msg)
            return error_msg

    def execute_silent_command(self, command: str) -> str:
        """function which execute command without generating any output

        Args:
            command (str): command passed for execution

        Returns:
            str: return success message if executed (not actual output of the command)
            otherwise error message if fail
        """
        try:
            LOG.info(f"command execution started {command}")
            subprocess.check_call(command, shell=True)
            LOG.debug(f"command executed succesfully : {command}")
            return "command executed succesfully"
        except subprocess.CalledProcessError as e:
            error_msg = f"Command execution failed with error: {e}"
            LOG.error(error_msg)
            return error_msg

    def run_command_obj(self, command: str) -> Union[subprocess.CompletedProcess, str]:
        """function take comand as string and return object as subprocess.completedprocess after successful execution

        Args:
            command (str): command passed for execution

        Returns:
            Union[subprocess.CompletedProcess, str]: return object if executed succesfully
            otherwise return error message as string
        """
        try:
            LOG.info(f"command execution started : {command}")
            result = subprocess.run(
                command,
                shell=True,
                stdout=subprocess.PIPE,
                stderr=subprocess.PIPE,
                text=True,
                check=False,
            )
            LOG.debug(f"command execution succesfully {command}")
            return result
        except subprocess.CalledProcessError as e:
            error_msg = f"Command execution failed with error: {e}"
            LOG.error(error_msg)
            return error_msg

    def execute_commands_sequentially(
        self, commands: List[str], mode: str = "output"
    ) -> Union[Dict[str, str], str]:
        """function to execute multiple commands with order in which they present in the list

        Args:
            commands (List[str]): multiple commands present in the list
            mode (str, optional): if mode selected to object
            it return object of commands present in the list Defaults to "output".

        Returns:
            Union[Dict[str, str], str]: if wrong mode passed it returns string as invalid parameter
            otherwise output or object mapped to command is returned in dictionary
        """

        command_output = {}

        if mode in ("output", "object"):
            for command in commands:
                if mode == "output":
                    command_output[command] = self.execute_command(command)
                else:
                    command_output[command] = self.run_command_obj(command)
            return command_output

        LOG.error("invalid parameter, it's either 'object' or 'output'")
        return "Invalid parameter"

    def execute_commands_in_parallel(
        self, commands: List[str], mode: str = "output", num_threads: int = 2
    ) -> Union[List[Union[str, subprocess.CompletedProcess]], str]:
        """function to execute multiple commands but using threadpoolexecutor
        so that it use threads to execute each commands parrallely

        Args:
            commands (List[str]): multiple commands present in the list
            mode (str, optional): if mode selected to object
            it return object of commands present in the list Defaults to "output".
            num_threads (int, optional): no of threads in which user want to execute default set to 2

        Returns:
            Union[List[Union[str, subprocess.CompletedProcess]], str]: invalid parameter returned if wrong mode passed
            otherwise object or output in form of string is returned
        """

        futures = []
        with concurrent.futures.ThreadPoolExecutor(max_workers=num_threads) as executor:
            if mode not in ("object", "output"):
                LOG.error("invalid parameter its either object or output")
                return "Invalid parameter"
            if mode == "output":
                futures = [executor.submit(self.execute_command, command) for command in commands]
            if mode == "object":
                futures = [executor.submit(self.run_command_obj, command) for command in commands]
            results = [future.result() for future in concurrent.futures.as_completed(futures)]
            return results

    def run_command_with_timeout(
        self, command: str, timeout_seconds: int
    ) -> Tuple[Union[str, None], Union[str, None], int]:
        """function to execute commands within the limited time passed as arguement

        Args:
            command (str): command as string
            timeout_seconds (int): maximum timeout as integer

        Returns:
            Tuple[Union[str, None], Union[str, None], int]: if executed return output as string,
            none in error and returncode as integer
            otherwise output as none returncode as -1 and error as timeout in case of failure
        """
        try:
            LOG.info(f"command execution started : {command}")
            result = subprocess.run(
                command,
                shell=True,
                stdout=subprocess.PIPE,
                stderr=subprocess.PIPE,
                text=True,
                timeout=timeout_seconds,
                check=False,
            )
            LOG.debug(f"command execution succesfully: {command}")
            return result.stdout, result.stderr, result.returncode
        except subprocess.TimeoutExpired:
            LOG.error(f"{command} : command timed out")
            return None, "Command timed out", -1

    def run_command_in_directory(
        self, command: str, directory: str
    ) -> Tuple[Union[str, None], Union[str, None], int]:
        """function which execute command in the specified directory provide by the user

        Args:
            command (str): command as string
            directory (str): directory in which user want to execute this command

        Returns:
            Tuple[Union[str, None], Union[str, None], int]: return standard output if executed succesfully
            error as none and returcode as integer otherwise error as string is returned
        """
        isExist = Path(directory).exists()
        if isExist:
            try:
                LOG.info(f"command execution started : {command}")
                result = subprocess.run(
                    command,
                    shell=True,
                    stdout=subprocess.PIPE,
                    stderr=subprocess.PIPE,
                    text=True,
                    cwd=directory,
                    check=False,
                )
                LOG.debug(f"command execution succesfuly : {command}")
                return result.stdout, result.stderr, result.returncode
            except subprocess.CalledProcessError as e:
                error_msg = f"Command execution failed with error: {e}"
                LOG.error(error_msg)
                return error_msg
        LOG.error(f"This directory doest found : {directory}")
        return f"This directory doest found : {directory}"

    def execute_command_for_piped(self, commands: List[str]) -> List[Optional[subprocess.Popen]]:
        """this is the method which take list of commands for input and return list of object of subprocess.Popen

        Args:
            commands (List[str]): multiple commnad as list

        Returns:
            List[Optional[subprocess.Popen]]:
        """
        processes = []

        for idx, command in enumerate(commands):
            try:
                stdin = processes[idx - 1].stdout if idx > 0 else None
                # pylint: disable=consider-using-with
                LOG.info(f"command execution started for command : {command}")
                process = subprocess.Popen(
                    command,
                    shell=True,
                    stdin=stdin,
                    stdout=subprocess.PIPE,
                    stderr=subprocess.PIPE,
                    text=True,
                )
                LOG.debug(f"command execution succsfully :{command}")
            except Exception as e:
                LOG.error(f"command execution failed with : {e}")

            if idx > 0:
                processes[idx - 1].stdout.close()
            processes.append(process)

        return processes

    def run_piped_commands(
        self, commands: List[str]
    ) -> Tuple[Union[str, None], Union[str, None], int]:
        """this is the method which used in execution of chaining commands where
        output of previous command act as input to next command after execution
        of first for loop output are extracted if command fails

        Args:
            commands (List[str]): multiple commands in a list

        Returns:
            Tuple[Union[str, None], Union[str, None], int]: return standard output if execute none otherwise,
            standard error if fail none otherwise and returncode 0 in success non zero otherwise
        """
        processes = self.execute_command_for_piped(commands)

        LOG.info("output processing start for the piped commands")
        stdout, stderr = "", ""
        for idx, process in enumerate(processes):
            output, error = process.communicate()
            retcode = process.poll()
            if retcode:
                error_msg = (
                    f"Command '{commands[idx]}' failed with exit status {retcode}"
                    if not error
                    else error
                )
                LOG.error(error_msg)
                return (
                    stdout,
                    error_msg,
                    retcode,
                )
            stdout = output
            stderr = error

        return stdout, stderr, 0
