""" utility to interact with the S3 bucket"""
import os
import uuid
import boto3

from pyneers.utils.logger import CustomLogger
from pyneers.utils.exceptions import (
    AwsConfigurationError,
    BucketNotFound,
    BucketConfigurationError,
    NoAvailableEvents,
)

LOG = CustomLogger.getLogger(__name__)


class S3:
    """S3 class to interact with the S3 bucket"""

    def __init__(self, bucket: str, profile: str = "default", useSession=False) -> None:
        LOG.info("Initializing s3 client")
        if useSession:
            session = boto3.Session(profile_name=profile)
            self.s3 = session.client("s3")
            self.__validate(session)
        else:
            self.s3 = boto3.client("s3")
        self.bucket = bucket
        self.paginator = self.s3.get_paginator("list_objects_v2")

        LOG.info("AWS Credentials Validation Complete")

    def __validate(self, session: boto3.Session) -> None:
        """Validate the AWS credentials and the bucket name"""
        sts = session.client("sts")
        try:
            sts.get_caller_identity()
        except Exception as e:
            raise AwsConfigurationError from e
        try:
            self.s3.list_objects_v2(Bucket=self.bucket)
        except Exception as e:
            raise BucketNotFound(self.bucket) from e

    def get_all_objects(self) -> list:
        """Get all objects in the bucket"""
        try:
            page_iterator = self.paginator.paginate(Bucket=self.bucket)
            LOG.info(f"Retrieved all objects in s3 bucket: {self.bucket}")
            OBJECTS = []
            for page in page_iterator:
                if "Contents" in page:
                    for obj in page["Contents"]:
                        OBJECTS.append(obj)
            return OBJECTS

        except Exception as e:
            raise BucketConfigurationError(self.bucket) from e

    def get_root_folder_structure(self) -> list:
        """Get the root folder structure in the bucket"""
        objects = self.get_all_objects()
        root = set()

        for obj in objects:
            key = obj["Key"]
            split_key = key.split("/")

            if split_key[0] == "" and len(split_key) > 1:
                root.add(split_key[1])
            if split_key[0] != "":
                root.add(split_key[0])
        root = list(root)
        LOG.info(f"Retrieved root folder structure in {self.bucket}")
        root.sort()
        return root

    def get_all_notifications(self):
        """Get all the event notifications for the bucket"""
        try:
            events = self.s3.get_bucket_notification_configuration(Bucket=self.bucket)
        except Exception:
            LOG.critical(
                f"No bucket found by the name: {self.bucket}",
                exc_info=True,
            )
        return events

    def get_topic_events(self):
        """Get all the topic event notifications for the bucket"""
        events = self.get_all_notifications()
        try:
            events = events.get("TopicConfigurations")
        except Exception as e:
            LOG.error("No Topic event notifications available", exc_info=True)
            raise NoAvailableEvents("TopicConfigurations") from e

        return events

    def get_queue_events(self):
        """Get all the queue event notifications for the bucket"""
        events = self.get_all_notifications()
        try:
            events = events.get("QueueConfigurations")
        except Exception as e:
            LOG.error("No Queue event notifications available", exc_info=True)
            raise NoAvailableEvents("QueueConfigurations") from e
        return events

    def get_lambda_events(self):
        """Get all the lambda event notifications for the bucket"""
        events = self.get_all_notifications()
        try:
            events = events.get("LambdaFunctionConfigurations")
        except Exception as e:
            LOG.error(
                "No Lambda Function event notifications available",
                exc_info=True,
            )
            raise NoAvailableEvents("LambdaFunctionConfigurations") from e
        return events

    def upload_file(self, local_path: str, object_path: str):
        """Upload a file to the bucket"""
        try:
            LOG.info("Uploading %s to %s:%s", local_path, self.bucket, object_path)
            self.s3.upload_file(Filename=local_path, Bucket=self.bucket, Key=object_path)
            LOG.info(
                "Finished uploading %s to %s:%s",
                local_path,
                self.bucket,
                object_path,
            )
        except Exception as e:
            raise e

    def rename_and_upload(self, local_path: str, object_path: str):
        """Rename the file and upload it to the bucket"""
        LOG.info("Renaming asset...")
        name = uuid.uuid4()
        oldfile = os.path.basename(local_path)
        ext = oldfile.split(".")[-1]
        newfile = f"{name}.{ext}"
        object_path = os.path.join(os.path.dirname(object_path), newfile)
        self.upload_file(local_path, object_path)

        return newfile

    def download_file(self, local_path: str, object_path: str):
        """Download a file from the bucket"""
        try:
            LOG.info("Downloading object from s3: %s", object_path)
            self.s3.download_file(Bucket=self.bucket, Key=object_path, Filename=local_path)
        except Exception as e:
            raise e
