"""
    Config Parser Utils Module
"""
from pathlib import Path
import configparser
from typing import Dict, Any, Union


def config_file_to_dict(file_path: Union[str, Path]) -> Dict[str, Dict[str, str]]:
    """
    Converts a multi-level .cfg or.ini config file to a Python dictionary.

    Args:
        file_path (str|Path): The path to the config file.

    Returns:
        dict: A dictionary representing the config file,
        with sections as keys and options as nested dictionaries.

    Example:
        file_path = "config.cfg"
        config_dict = config_file_to_dict(file_path)
        print(config_dict)
    """

    # Create an instance of ConfigParser
    config = configparser.ConfigParser()

    # Read and parse the config file
    config.read(str(file_path))

    # Create an empty dictionary to store the config values
    config_dict: Dict[str, Union[Any, Dict[str, Any]]] = {}

    # Iterate over each section in the config file
    for section in config.sections():
        # Create a nested dictionary for each section
        section_dict: Dict[str, Union[Any, Dict[str, Any]]] = {}

        # Iterate over each option in the section
        for option in config.options(section):
            # Retrieve the value of the option and add it to the nested dictionary
            section_dict[option] = config.get(section, option)

        # Add the nested dictionary to the main config dictionary
        config_dict[section] = section_dict

    # Return the resulting config dictionary
    return config_dict
