""" Handler for CSV files handling using Pandas """
import csv
from functools import lru_cache
import pandas as pd
from pyneers.utils.base_handler import BaseHandler
from pyneers.utils.logger import CustomLogger
from pyneers.utils.exceptions import FileWriteError, FileReadError

LOG = CustomLogger.getLogger(__name__)


class CSVHandler(BaseHandler):
    """Class responsible for handling CSV files"""

    @lru_cache(maxsize=128)
    def cache_read(self, use_pandas=False, encoding="utf-8") -> pd.DataFrame:
        """Method to read CSV files using cache and return a Pandas DataFrame.

        Args:
            encoding (str, optional): Type of encoding to be used. Defaults to "utf-8".

        Raises:
            FilePathError: Raises FilePathError if file path is incorrect and does not exist.
            FileSizeError: Raises FileSizeError if file size is more than 500 MB.
            FileTypeError: Raises FileTypeError if file type is incorrect.
            FileReadError: Raises FileReadError if it is unable to open or read the file.

        Returns:
            pd.DataFrame: Returns a Pandas DataFrame containing the CSV data.
        """
        self.validate_file()
        try:
            if use_pandas:
                return pd.read_csv(
                    self.file_path,
                    encoding=encoding,
                )

            with open(self.file_path, "r", encoding=encoding) as f:
                reader = csv.reader(f)
                return list(reader)

        except Exception as e:
            LOG.error(e)
            raise FileReadError() from e

    def read(self, use_pandas=False, encoding="utf-8") -> pd.DataFrame:
        """Method to read CSV files and return a Pandas DataFrame.

        Args:
            encoding (str, optional): Type of encoding to be used. Defaults to "utf-8".

        Raises:
            FilePathError: Raises FilePathError if file path is incorrect and does not exist.
            FileSizeError: Raises FileSizeError if file size is more than 500 MB.
            FileTypeError: Raises FileTypeError if file type is incorrect.
            FileReadError: Raises FileReadError if it is unable to open or read the file.

        Returns:
            pd.DataFrame: Returns a Pandas DataFrame containing the CSV data.
        """
        self.validate_file()
        try:
            if use_pandas:
                return pd.read_csv(
                    self.file_path,
                    encoding=encoding,
                )

            with open(self.file_path, "r", encoding=encoding) as f:
                reader = csv.reader(f)
                return list(reader)

        except Exception as e:
            LOG.error(e)
            raise FileReadError() from e

    def write(self, data: str, mode="w", encoding="utf-8") -> None:
        """Method to write to text files

        Args:
            data (str): Data to be written to the file
            mode (str, optional): Mode for writing to the file. Uses "w" mode to write to the file
              and "a" mode to append to the file. Defaults to "w".
            encoding (str, optional): Type of encoding to be used. Defaults to "utf-8".

        Raises:
            FilePathError: Raises FilePathError if file path is incorrect and does not exist
            FileSizeError: Raises FileSizeError if file size is more than 500 MB
            FileTypeError: Raises FileTypeError if file type is incorrect
            FileWriteError: Raises FileWriteError if it is unable to open or write to a file

        Returns:
            None: Returns nothing in case of a successful write
        """
        self.validate_file()
        try:
            with open(self.file_path, mode, encoding=encoding) as f:
                writer = csv.writer(f, quoting=csv.QUOTE_NONE)
                writer.writerows(data)

        except Exception as e:
            LOG.error(e)
            raise FileWriteError() from e
