"""
Date util module
"""
# pylint: disable = E0401
import pendulum


class DateTimeUtils:
    """
    Class for handling various date and time operations
    """

    @staticmethod
    def get_seconds_from_time(time_str: str) -> float:
        """
        Get seconds from time string in any format
        :param time_str: time value in any format
        :return: seconds in float format
        """
        try:
            time_obj = pendulum.parse(time_str).time()
            total_seconds = time_obj.hour * 3600 + time_obj.minute * 60 + time_obj.second
            return total_seconds
        except Exception as e:
            raise ValueError(f"Error converting time string to seconds: {e}") from e

    @staticmethod
    def convert_to_standard_date(date_string: str, format: str = "YYYY-MM-DD") -> str:
        """
        Convert any type of input date to 'YYYY-MM-DD' format
        :param date_string: date value in str format
        :param format: format string for the date
        :return: date in 'YYYY-MM-DD' format
        """
        try:
            date_obj = pendulum.parse(date_string, strict=False, exact=False, format=format)
            return date_obj.format(format)
        except Exception as e:
            raise ValueError(f"Error converting date string to standard format: {e}") from e

    @staticmethod
    def convert_to_iso_datetime(date_string: str) -> str:
        """
        Convert any type of input date to 'YYY-MM-DDTHH:MM:SS' format
        :param date_string: date string
        :return: 'YYY-MM-DDTHH:MM:SS' format date
        """
        try:
            date_obj = pendulum.parse(date_string, strict=False, exact=False)
            return date_obj.format("YYYY-MM-DDTHH:mm:ss")
        except Exception as e:
            raise ValueError(f"Error converting date string to ISO datetime format: {e}") from e

    @staticmethod
    def get_current_datetime(format: str = "YYYY-MM-DDTHH:mm:ss.SSS") -> str:
        """
        Return present time in the format specified. Default format is 'YYYY-MM-DDTHH:MM:SS.FF'
        :param format: format string for the date time
        :return: date time in the specified format
        """
        try:
            return pendulum.now().format(format)
        except Exception as e:
            raise ValueError(f"Error getting current datetime: {e}") from e

    @staticmethod
    def add_delay_to_datetime(
        datetime_str: str,
        delay_minutes: int = 0,
        delay_hours: int = 0,
        delay_days: int = 0,
        delay_seconds: int = 0,
    ) -> str:
        """
        Add a delay to a specific date and time.
        :param datetime_str: date and time string in any format
        :param delay_minutes: number of minutes to delay the datetime (default: 0)
        :param delay_hours: number of hours to delay the datetime (default: 0)
        :param delay_days: number of days to delay the datetime (default: 0)
        :param delay_seconds: number of seconds to delay the datetime (default: 0)
        :return: delayed date and time string
        """
        try:
            datetime_obj = pendulum.parse(datetime_str)
            delayed_datetime = datetime_obj.add(
                minutes=delay_minutes, hours=delay_hours, days=delay_days, seconds=delay_seconds
            )
            return delayed_datetime.format("YYYY-MM-DDTHH:mm:ss")
        except Exception as e:
            raise ValueError(f"Error adding delay to datetime: {e}") from e

    @staticmethod
    async def async_get_seconds_from_time(time_str: str) -> float:
        """
        Async version of same function
        """
        return DateTimeUtils.get_seconds_from_time(time_str)

    @staticmethod
    async def async_convert_to_standard_date(date_string: str, format: str = "YYYY-MM-DD") -> str:
        """
        Async version of same function
        """
        return DateTimeUtils.convert_to_standard_date(date_string, format)

    @staticmethod
    async def async_convert_to_iso_datetime(date_string: str) -> str:
        """
        Async version of same function
        """
        return DateTimeUtils.convert_to_iso_datetime(date_string)

    @staticmethod
    async def async_get_current_datetime(format: str = "YYYY-MM-DDTHH:mm:ss.SSS") -> str:
        """
        Async version of same function
        """
        return DateTimeUtils.get_current_datetime(format)

    @staticmethod
    async def async_add_delay_to_datetime(
        datetime_str: str,
        delay_minutes: int = 0,
        delay_hours: int = 0,
        delay_days: int = 0,
        delay_seconds: int = 0,
    ) -> str:
        """
        Async version of same function
        """
        return DateTimeUtils.add_delay_to_datetime(
            datetime_str, delay_minutes, delay_hours, delay_days, delay_seconds
        )
