"""
    Base DB Handler: Chooses either the SQL or the MongoDB context manager based on db_type
"""
from abc import ABC, abstractmethod
import configparser
from typing import Literal

from pydantic import BaseModel, model_validator

from pyneers.utils.logger import CustomLogger


LOG = CustomLogger.getLogger(__name__)


class DBConfig(BaseModel):
    """
    Pydantic model for creating db config

    Args:
        Username, Password, Host, Port, Database Name, Database Type,
        Authentication Source(Mongo Specific Users), Authentication Mechanism(Mongo Specific Users)
    """

    username: str
    password: str
    host: str
    port: str
    database_name: str
    db_type: Literal["postgresql", "mongodb", "mysql"]
    auth_source: str = None
    auth_mechanism: str = None

    @model_validator(mode="after")
    def validate_mongodb_fields(self):
        """
        Validate MongoDB-specific fields for connecting to a MongoDB database.

        Raises:
            ValueError: If the database type is MongoDB and either the
                'auth_source' or 'auth_mechanism' fields are empty or None.

        Returns:
            None
        """
        db_type = self.db_type
        auth_source = self.auth_source
        auth_mechanism = self.auth_mechanism

        if db_type == "mongodb" and (
            auth_source == ""
            or auth_source is None
            or auth_mechanism == ""
            or auth_mechanism is None
        ):
            raise ValueError(
                "Authorization Source and Authorization Mechanism are necessary"
                "for connecting to a MongoDB database"
            )
        return self


class BaseDBHandler(ABC):
    """
    Initialize the database configuration.

    Args:
        db_config (DBConfig): A Pydantic model representing the database configuration.
        config_file (str): Path to the config file (optional).
        max_retries (int): Maximum number of retries to establish the database connection (default: 3).
        retry_interval (float): Time interval (in seconds) between each retry (default: 1 second).
    """

    def __init__(
        self,
        db_config: DBConfig,
        config_file: str = None,
        max_retries: int = 3,
        retry_interval: float = 1,
    ):
        self.max_retries = max_retries
        self.retry_interval = retry_interval
        self.db_url = None

        self.db_url_formats = {
            "postgresql": "{db_type}://{username}:{password}@{host}:{port}/{database_name}",
            "mongodb": "{db_type}://{username}:{password}@{host}:{port}/",
            "mysql": "{db_type}+mysqlconnector://{username}:{password}@{host}:{port}/{database_name}",
        }

        if config_file:
            config = configparser.ConfigParser()
            config.read(config_file)

            db_config.username = config.get("DATABASE", "username", fallback=db_config.username)
            db_config.password = config.get("DATABASE", "password", fallback=db_config.password)
            db_config.host = config.get("DATABASE", "host", fallback=db_config.host)
            db_config.port = config.get("DATABASE", "port", fallback=db_config.port)
            db_config.database_name = config.get(
                "DATABASE", "database_name", fallback=db_config.database_name
            )

            if db_config.database_name == "mongodb":
                db_config.auth_source = config.get(
                    "DATABASE", "authSource", fallback=db_config.auth_source
                )
                db_config.auth_mechanism = config.get(
                    "DATABASE", "authMechanism", fallback=db_config.auth_mechanism
                )

        db_type = db_config.db_type
        format_string = self.db_url_formats.get(db_type)

        if format_string:
            self.db_url = format_string.format(
                db_type=db_type,
                username=db_config.username,
                password=db_config.password,
                host=db_config.host,
                port=db_config.port,
                database_name=db_config.database_name,
            )
        else:
            LOG.error("Unsupported database type")

        @abstractmethod
        def create_session():
            pass

        @abstractmethod
        def _retry_query():
            pass

        @abstractmethod
        def _execute_query():
            pass

        @abstractmethod
        def heartbeat_check():
            pass

        @abstractmethod
        def conection_monitor():
            pass

        @abstractmethod
        def start_connection_monitoring():
            pass

        @abstractmethod
        def close_connection():
            pass
