"""
DbUtils Module for a MongoDB database
"""
import threading
import time
import urllib.parse
from typing import Union
from pymongo import MongoClient
from pyneers.utils.logger import CustomLogger
from pyneers.utils.db.db_handler import DBConfig, BaseDBHandler
from pyneers.utils.db.db_operations_handler import MongoQueryObject, MongoOperationHandler

LOG = CustomLogger.getLogger(__name__)


# pylint: disable=duplicate-code
class DBUtilsMongo(BaseDBHandler):
    """
    Database utility class for making database connections using pymongo.
    Args:
        db_config: Instance of class DBConfig containing database configuration properties
        config_file: File containing database configuration properties
    """

    def __init__(
        self,
        db_config: DBConfig,
        config_file: str = None,
    ):
        super().__init__(db_config, config_file)

        self.database_name = db_config.database_name
        self.server_selection_timeout_ms = 3000
        self.socket_selection_timeout_ms = 5000

        retries = 0
        while retries < self.max_retries:
            try:
                LOG.info(f"Establishing database connection (Attempt {retries}/{self.max_retries})")
                self.client = MongoClient(
                    db_config.host,
                    port=int(db_config.port),
                    username=urllib.parse.quote_plus(db_config.username),
                    password=urllib.parse.quote_plus(db_config.password),
                    authSource=db_config.auth_source,
                    authMechanism=db_config.auth_mechanism,
                    serverSelectionTimeoutMS=self.server_selection_timeout_ms,
                    socketTimeoutMS=self.socket_selection_timeout_ms,
                )
                result = self.client.admin.command("ping")
                if result["ok"] == 1:
                    # Create a client session that may be used only by the Mongo Client that started it
                    self.Session = self.client.start_session(causal_consistency=True)
                    LOG.info("Database connection Successful")
                # Connection successful, exit the loop
                break
            except Exception as ex:
                retries += 1
                LOG.info(
                    f"Failed to establish the database connection (Attempt {retries}/{self.max_retries}). Error:{ex}."
                )
                time.sleep(self.retry_interval)

        if retries == self.max_retries:
            raise ConnectionError(
                "Failed to establish the database connection after maximum retries."
            )

    def create_session(self) -> Union[MongoClient, MongoClient.start_session]:
        """
        Create a new database session.
        Returns:
            Client and Session object for a MongoDB instance.
        """
        return self.client, self.Session

    def _retry_query(
        self, queryObject: MongoQueryObject, read_pandas: bool = False
    ) -> Union[
        MongoOperationHandler._execute_find,
        MongoOperationHandler._execute_insert_one,
        MongoOperationHandler._execute_delete_many,
        MongoOperationHandler._execute_update_many,
    ]:
        """
        Method to retry a MongoDB query.
        Raises:
            Exception: When the query specifed cannot be executed
        Args:
            Instance of the MongoQueryObject class
        """

        with self.client.start_session(causal_consistency=True) as session:
            with session.start_transaction():
                try:
                    operation_handler = MongoOperationHandler(self.client, self.database_name)
                    return operation_handler.execute_operation(queryObject, read_pandas)

                except Exception as ex:
                    LOG.error(f"Failed to execute the query. Error: {ex}")
                    return None

    def execute_query(self, queryObject: MongoQueryObject, read_pandas: bool = False):
        """
        Method to execute a mongoDB query
        Args:
            Instance of the MongoQueryObject class
        """
        LOG.info(f"Executing DB Query:{queryObject}")
        return self._retry_query(queryObject, read_pandas)

    def heartbeat_check(self) -> bool:
        """
        Perform a heartbeat check to verify the database connection.
        Returns:
            bool: True if the heartbeat query is successful, False otherwise.
        """
        try:
            LOG.info("Starting DB Heartbeat Check")
            result = self.client.admin.command("ping")
            if result["ok"] == 1:
                LOG.info("DB Heartbeat Check Successful")
                return True
        except Exception as ex:
            LOG.info(f"DB Heartbeat Check Failed : Error: {ex}")
            return False

        return True

    def _connection_monitor(self, interval=10) -> None:
        """
        Periodically monitor the database connection and retry if it's lost.
        Args:
            interval (int): Interval (in seconds) between each check (default: 10 seconds).
        """
        while True:
            LOG.info("Starting DB connection monitor in separate thread")
            if not self.heartbeat_check():
                LOG.info("Database connection is lost. Retrying...")
                # Create a client session that may be used only by the Mongo Client that started it
                self.Session = self.client.start_session(causal_consistency=True)
                LOG.info("Database connection is restored.")

            time.sleep(interval)

    def start_connection_monitoring(self, interval=10) -> None:
        """
        Start the connection monitoring in a separate thread.
        Args:
            interval (int): Interval (in seconds) between each check (default: 10 seconds).
        """
        monitor_thread = threading.Thread(target=self._connection_monitor, args=(interval,))
        monitor_thread.daemon = True
        monitor_thread.start()

    def close_connection(self) -> None:
        """
        Close the database connection.
        """
        LOG.info("Closing the DB Connection")
        self.client.close()

    def __enter__(self) -> "DBUtilsMongo":
        """
        enter
        """
        self.start_connection_monitoring()
        return self

    def __exit__(self, exc_type, exc_value, traceback) -> None:
        """
        exit
        """
        self.close_connection()


def connect_to_db(db_config: DBConfig) -> tuple:
    """
    Function: Helper Method to connect to database. To be used while importing this module
    Args:
        DBConfig
    """
    db_utils = DBUtilsMongo(db_config=db_config)
    return db_utils.create_session()
