"""
Module for carrying out database operations
"""
from typing import Literal, Union
import sqlalchemy
import pandas as pd
from sqlalchemy import Row, text
from pymongo import MongoClient
import sqlparse
from pyneers.utils.logger import CustomLogger

LOG = CustomLogger.getLogger(__name__)


class SQLQueryHandler:
    """
    A class for executing SQL queries using SQLAlchemy.

    This class provides methods to execute SQL queries and handles various types of SQL statements,
    including SELECT, INSERT, UPDATE, and DELETE.

    Args:
    engine (sqlalchemy.engine.base.Engine): An SQLAlchemy database engine.

    Attributes:
    engine (sqlalchemy.engine.base.Engine): The SQLAlchemy database engine to
    be used for query execution.

    Methods:
    execute_operation(query: str, params=None) -> Union[List[Row], int, None]:
    Execute an SQL query with optional parameters and return the result.

    """

    def __init__(self, engine: sqlalchemy.engine.base.Engine):
        self.engine = engine

    def execute_operation(
        self, query: str, read_pandas: bool = False, params=None
    ) -> Union[list[Row], int, None]:
        """
        Execute an SQL query with optional parameters and return the result.

        Args:
        query (str): The SQL query to be executed.
        params (dict, optional): Parameters to be passed to the SQL query, if any.

        Returns:
        Union[List[Row], int, None]: The result of the SQL query. For SELECT queries,
        it returns a list of rows.
        For INSERT, UPDATE, and DELETE queries, it returns the number of affected rows.
        Returns None if the query is invalid or if an error occurs during execution.
        """

        queryString = str(query)  # noqa: FURB123
        result = None
        if isinstance(query, str):
            query = text(query)

        with self.engine.connect() as conn:
            try:
                query = text(queryString)  # Use parameterized queries for safety
                query_type = sqlparse.parse(queryString)[0].get_type().upper()
                # result = conn.execute(query, params)
                # result = list(result) if query_type == "SELECT" else result

                if str(query_type) == "SELECT":
                    # for row in result:
                    #     LOG.debug(f"row:{row}")
                    #     resultList.append(row)
                    if read_pandas:
                        result = pd.read_sql(query, self.engine)
                    else:
                        result = conn.execute(query, params)
                        result = list(result)

                else:
                    result = conn.execute(query, params)
                    conn.commit()

                    if query_type in ("INSERT", "UPDATE", "DELETE"):
                        # For INSERT, UPDATE, and DELETE queries, consider checking
                        # if the affected rowcount is greater than 0.
                        affected_rows = result.rowcount
                        if affected_rows > 0:
                            LOG.info(f"Query Executed Successfully. {affected_rows} rows affected.")
                        else:
                            LOG.error("Query Executed Unsuccessfully. No rows affected.")

                    else:
                        # For other query types (e.g., SELECT), you can log
                        # success based on the query execution.
                        LOG.info("Query Executed Successfully.")

            except Exception as e:
                # Handle any exceptions, such as SQL syntax errors
                LOG.error(f"Query execution failed: {e}")

            return result


class MongoQueryObject:
    """
    Class for creating a query object to execute MongoDB queries.
    Args:
    collection_name : Name of the database collection
    operation : The specific CRUD operation that is needed
    query : Database query to be executed
    data : The document to be inserted into the specified collection
    projections : Fields to include and exclude from the query result
    """

    # pylint: disable=R0913
    def __init__(
        self,
        collection_name: str,
        operation: Literal["find", "insert", "update", "delete"],
        query: dict = None,
        data: dict = None,
        projections: dict = None,
    ):
        self.collection_name = collection_name
        self.operation = operation
        self.query = query
        self.data = data
        self.projections = projections


class MongoOperationHandler:
    """
    Class for creating a query object to execute MongoDB queries.
    Args:
    client: An instance of MongoClient containing all the connection properties
    database_name: The name of the database to carry out the queries
    collection: The name of the collection in a particular database
    Methods:
    _execute_find() -> implementation to find documents
    _execute_insert()_one -> implementation to insert a document
    _execute_update()_many -> implementation to update a document
    _execute_delete()_many -> implementation to delete a document
    """

    def __init__(self, client: MongoClient, database_name: str):
        self.client = client
        self.database_name = database_name
        self.collection = None

    def execute_operation(self, queryObject: MongoQueryObject, read_pandas: bool = None):
        """
        Method for executing a MongoDB query
        Args: An instance of the MongoQueryObject
        Returns:
        The output based on operation type
        """
        db = self.client.get_database(self.database_name)
        self.collection = db[queryObject.collection_name]
        operation = queryObject.operation

        if operation == "find":
            return self._execute_find(queryObject, read_pandas)
        if operation == "insert":
            return self._execute_insert_one(queryObject)
        if operation == "update":
            return self._execute_update_many(queryObject)
        if operation == "delete":
            return self._execute_delete_many(queryObject)

        LOG.error("Unsupported operation")
        return None

    # pylint: disable=R1705
    def _execute_find(self, queryObject: MongoQueryObject, read_pandas: bool = False):
        if queryObject.query is None:
            LOG.error("Query attribute must be specified for running a find operation")
            return None
        result = self.collection.find(queryObject.query, queryObject.projections)
        if result is not None:
            LOG.info(f"Query Execution successful with result: {result}")
            if read_pandas:
                result = pd.DataFrame.from_records(result)
            return result
        else:
            LOG.info(f"No Documents are found for the query: {queryObject.query}")
        return None

    # pylint: disable=R1705
    def _execute_insert_one(self, queryObject: MongoQueryObject):
        if queryObject.data is None:
            LOG.error("Data attribute must be specified for running a insert operation")
            return None
        inserted_result = self.collection.insert_one(queryObject.data)
        if inserted_result.acknowledged:
            LOG.info(f"Document Insertion successful with result ID: {inserted_result.inserted_id}")
            return inserted_result.inserted_id
        else:
            LOG.info(f"The following Document was not inserted successfully: {queryObject.data}")
        return inserted_result

    def _execute_update_many(self, queryObject: MongoQueryObject):
        if queryObject.data is None or queryObject.query is None:
            LOG.error("Query and Data attributes must be specified for running a update operation")
            return None
        update_result = self.collection.update_many(queryObject.query, {"$set": queryObject.data})
        if update_result.modified_count == 0:
            LOG.info(f"Update is Unsuccessful. Modified {update_result.modified_count} documents.")
        else:
            LOG.info(f"Update successful. Modified {update_result.modified_count} documents.")
        return update_result

    def _execute_delete_many(self, queryObject: MongoQueryObject):
        if queryObject.query is None:
            LOG.error("Query attribute must be specified for running a delete operation")
            return None
        delete_result = self.collection.delete_many(queryObject.query)
        if delete_result.deleted_count == 0:
            LOG.info("Delete Unsuccessful. No documents were deleted.")
        else:
            LOG.info(f"Delete successful. Deleted {delete_result.deleted_count} documents.")
        return delete_result
