"""
DBUtils Module for SQL Databases (Postgresql and MySQl)
"""
import threading
import time
import sqlalchemy
from sqlalchemy import create_engine, text
from sqlalchemy.orm import sessionmaker
from pyneers.utils.db.db_handler import BaseDBHandler
from pyneers.utils.db.db_handler import DBConfig
from pyneers.utils.logger import CustomLogger
from pyneers.utils.db.db_operations_handler import SQLQueryHandler

LOG = CustomLogger.getLogger(__name__)


class DBUtilsSql(BaseDBHandler):
    """
    Database utility class for making database connections using SQLAlchemy.
    Args:
        db_config: Instance of class DBConfig containing database configuration properties
        config_file: File containing database configuration properties
    """

    def __init__(self, db_config: DBConfig, config_file: str = None):
        super().__init__(db_config, config_file)
        self.heartbeat_query = text("SELECT 1;")

        retries = 0
        while retries < self.max_retries:
            try:
                LOG.info(f"Establishing database connection (Attempt {retries}/{self.max_retries})")
                self.engine = create_engine(self.db_url)
                self.Session = sessionmaker(bind=self.engine)
                LOG.info("Database connection Successful")
                # Connection successful, exit the loop
                break
            except Exception as ex:
                retries += 1
                LOG.info(
                    f"Failed to establish the database connection (Attempt {retries}/{self.max_retries}). Error:{ex}."
                )
                time.sleep(self.retry_interval)

        if retries == self.max_retries:
            raise ConnectionError(
                "Failed to establish the database connection after maximum retries."
            )

    def create_session(self):
        """
        Create a new database session.

        Returns:
            sqlalchemy.orm.Session: A new database session object.
        """

        return self.Session()

    def _retry_query(
        self, query, read_pandas: bool = False, params=None
    ) -> sqlalchemy.CursorResult:
        """
        Retry the query execution in case of a connection issue.

        Args:
            query (str or sqlalchemy.text): The raw SQL query to execute.
            params (dict): Dictionary containing the parameters for the query (optional).
            max_retries (int): Maximum number of retries (default: 3).
            retry_interval (float): Time interval (in seconds) between each retry (default: 1 second).

        Raises:
            Exception if the specified query cannot be executed on the database
            Connection Error : If the connection to the database is lost

        Returns:
            Any: The result of the query execution.
        """
        if isinstance(query, str):
            query = text(query)

        retries = 0
        while retries < self.max_retries:
            sqlQueryHandler = SQLQueryHandler(self.engine)
            return sqlQueryHandler.execute_operation(query, read_pandas, params)

        raise ConnectionError("Failed to execute the query after maximum retries.")

    def execute_query(
        self, query, read_pandas: bool = False, params=None
    ) -> sqlalchemy.CursorResult:
        """
        Execute a parameterized SQL query on the database.

        Args:
            query (str or sqlalchemy.text): The raw SQL query to execute.
            params (dict): Dictionary containing the parameters for the query (optional).

        Returns:
            Any: The result of the query execution.
        """
        LOG.info(f"Executing DB Query:{query}, Params:{params}")
        return self._retry_query(query, read_pandas, params)

    def heartbeat_check(self) -> bool:
        """
        Perform a heartbeat check to verify the database connection.

        Raises:
            Connection Error if the database is disconnected
        Returns:
            bool: True if the heartbeat query is successful, False otherwise.
        """

        try:
            LOG.info("Starting DB Heartbeat Check")
            self._retry_query(self.heartbeat_query)
            LOG.info("DB Heartbeat Check Successful")
            return True
        except ConnectionError:
            LOG.info("DB Heartbeat Check Failed")
            return False

    def _connection_monitor(self, interval=10) -> None:
        """
        Periodically monitor the database connection and retry if it's lost.

        Args:
            interval (int): Interval (in seconds) between each check (default: 10 seconds).
        """
        while True:
            LOG.info("Starting DB connection monitor in separate thread")
            if not self.heartbeat_check():
                LOG.info("Database connection is lost. Retrying...")
                self._retry_query(self.heartbeat_query)
                LOG.info("Database connection is restored.")

            time.sleep(interval)

    def start_connection_monitoring(self, interval=10) -> None:
        """
        Start the connection monitoring in a separate thread.

        Args:
            interval (int): Interval (in seconds) between each check (default: 10 seconds).
        """
        monitor_thread = threading.Thread(target=self._connection_monitor, args=(interval,))
        monitor_thread.daemon = True
        monitor_thread.start()

    def close_connection(self) -> None:
        """
        Close the database connection.
        """
        LOG.info("Closing the DB Connection")
        self.engine.dispose()

    def __enter__(self) -> "DBUtilsSql":
        """
        enter
        """
        self.start_connection_monitoring()
        return self

    def __exit__(self, exc_type, exc_value, traceback) -> None:
        """
        exit
        """
        # pylint: disable=W0613
        self.close_connection()


def connect_to_db(db_config: DBConfig) -> sessionmaker:
    """
    Function: Helper Method to connect to database. To be used while importing this module

    Args:
        DBConfig
    """

    db_utils = DBUtilsSql(db_config=db_config)
    return db_utils.create_session()
