"""
ExcelHandler Module

This module defines the ExcelHandler class, responsible for handling Excel files.
It provides methods for reading and writing data to Excel files using the Pandas library.

Classes:
    - ExcelHandler: Class for Excel file handling.

Methods:
    - read(sheet_name=None): Read data from an Excel file.
    - write(sheet_name, force=False, data): Write data to an Excel file,
      optionally appending to an existing sheet.

Exceptions:
    - FileReadError: Raised if an error occurs while reading the Excel file.
"""
import pandas as pd
from pyneers.utils.base_handler import BaseHandler
from pyneers.utils.logger import CustomLogger
from pyneers.utils.exceptions import FileReadError

LOG = CustomLogger.getLogger(__name__)


class ExcelHandler(BaseHandler):
    """Class responsible for handling Excel files"""

    def read(self, sheet_name=None) -> pd.DataFrame:
        """
        Read data from an Excel file.

        Args:
            sheet_name (str, optional): The name of the sheet to read. Defaults to None.

        Returns:
            pd.DataFrame: A Pandas DataFrame containing the data from the specified sheet.

        Raises:
            FileReadError: If an error occurs while reading the Excel file.
        """
        self.validate_file()
        try:
            return pd.read_excel(self.file_path, sheet_name=sheet_name, engine="openpyxl")

        except Exception as e:
            LOG.error(e)
            raise FileReadError() from e

    # pylint: disable= arguments-differ
    def write(self, sheet_name: str, force: False, data: dict):
        """
        Write data to an Excel file, optionally appending to an existing sheet.

        Args:
            sheet_name (str): The name of the sheet to write to.
            force (bool, optional): If True, overwrite the existing sheet with the new data.
                If False (default), append the new data to the existing sheet.
            data (dict): The data to be written as a dictionary.

        Raises:
            ValueError: If 'force' is False and the columns of
            the new data do not match the columns in the current sheet.
        """
        self.validate_file()
        df = self.read(sheet_name=sheet_name)
        # Create a DataFrame from the dict
        new_df = pd.DataFrame(data, index=[0])
        result_df = None

        if force:
            result_df = new_df

        else:
            old_columns = set(df.columns)
            new_columns = set(new_df.columns)
            if not new_columns.issubset(old_columns):
                LOG.error(
                    "The columns of the row to be appended does not match the columns in the current sheet"
                )
                raise ValueError(
                    " Row cannot be appended to the excel file as there is a mismatch of columns"
                )

            # Append df2 as a row to df1
            result_df = pd.concat([df, new_df], ignore_index=True)

        print(result_df)

        # pylint: disable = abstract-class-instantiated
        with pd.ExcelWriter(
            self.file_path, mode="a", engine="openpyxl", if_sheet_exists="replace"
        ) as writer:
            # Write the data to the Excel file, specifying the sheet name and index option
            result_df.to_excel(writer, sheet_name=sheet_name, index=False)
