"""Exceptions for utilities"""
import json

from typing import Dict, Optional


class FilePathError(Exception):
    """method to raise exception when there is an error in file path"""

    def __init__(self) -> None:
        msg = "File path incorrect or File does not exist!"
        super().__init__(msg)


class FileSizeError(Exception):
    """method to raise exception when file size exceeds defined limit"""

    def __init__(self) -> None:
        msg = "File size greater than 500 MB!"
        super().__init__(msg)


class FileTypeError(Exception):
    """method to raise exception when there is unsupported file format"""

    def __init__(self) -> None:
        msg = "File format not supported"
        super().__init__(msg)


class FileReadError(Exception):
    """Method to raise exception when there is an error while reading a file."""

    def __init__(self, msg="Error: unable to read file"):
        super().__init__(msg)


class FileWriteError(Exception):
    """method to raise exception when there is an error while writing a file"""

    def __init__(self) -> None:
        msg = "Error: unable to write file"
        super().__init__(msg)


class FileLoadError(Exception):
    """method to raise exception when there is an error while loading a file."""

    def __init__(self, msg="Error: unable to load file"):
        super().__init__(msg)


class PollingTimeout(Exception):
    """method to raise exception when polling timeout occurs"""

    def __init__(self, mins: int, err: Optional[Dict] = None) -> None:
        super().__init__(
            f"Failed after polling for {mins} minutes \nerror: {json.dumps(err, indent=3)}"
        )


class ArgoCDAPIError(Exception):
    """method to raise exception when argo cd api fails"""

    def __init__(self, msg: str) -> None:
        super().__init__(msg)


class AwsException(Exception):
    """method to raise exception when aws api fails"""


class BucketNotFound(AwsException):
    """method to raise exception when bucket is not found"""

    def __init__(self, name: str) -> None:
        super().__init__(f"Bucket {name} not found")


class BucketConfigurationError(AwsException):
    """method to raise exception when bucket configuration is invalid"""

    def __init__(self, name: str) -> None:
        super().__init__(f"Bucket {name} is empty")


class NoAvailableEvents(AwsException):
    """method to raise exception when no events are available"""

    def __init__(self, type: str) -> None:
        super().__init__(f"{type} as no available events")


class AwsConfigurationError(AwsException):
    """method to raise exception when aws configuration is invalid"""

    def __init__(self) -> None:
        super().__init__("AWS Access Tokens are invalid")


class GithubException(Exception):
    """method to raise exception when github api fails"""


class BadCredentials(GithubException):
    """method to raise exception when github credentials are invalid"""

    def __init__(self) -> None:
        super().__init__("Github Credentials are Invalid")


class PRMergeError(GithubException):
    """method to raise exception when github credentials are invalid"""

    def __init__(self) -> None:
        super().__init__("Unable to merge PR")


class RepositoryDoesNotExist(GithubException):
    """method to raise exception when repository does not exist"""

    def __init__(self, repo: str) -> None:
        super().__init__(f"Repository {repo} not found")


class KubernetesException(Exception):
    """method to raise exception when kubernetes api fails"""


class JobRunning(KubernetesException):
    """method to raise exception when job is still in running state"""

    def __init__(self, name: str) -> None:
        super().__init__(f"{name} job is still in running state")
