# pylint: skip-file
""" Github utils to clone, commit, push, create PR and merge PR """
import os
import time
import shutil
import github

from git.repo import Repo
from github import PullRequest

from pyneers.utils.logger import CustomLogger
from pyneers.utils.exceptions import BadCredentials, PRMergeError, RepositoryDoesNotExist

LOG = CustomLogger.getLogger(__name__)


class Github:
    """Github class to clone, commit, push, create PR and merge PR"""

    def __init__(
        self,
        repo_name: str,
        username: str,
        password: str,
        working_dir: str | None = None,
        org: str | None = "amagimedia",
    ) -> None:
        self.username = username
        self.password = password
        self.working_dir = working_dir if working_dir is not None else "."
        self.repo_name = repo_name
        self.org = org
        self.repo_path = os.path.join(self.working_dir, self.repo_name)
        self.remote = f"https://{username}:{password}@github.com/{org}/{repo_name}.git"
        self.github = self.__setup_and_validate()

        LOG.info("Github configuration complete for %s/%s", self.org, self.repo_name)

    def __setup_and_validate(self) -> github.Github:
        """Method to setup and validate github credentials and repository"""
        try:
            gh = github.Github(self.username, self.password)
            repo_name = f"{self.org}/{self.repo_name}"
            gh.get_repo(repo_name)
        except github.BadCredentialsException as e:
            LOG.error("Github credentials are not valid", exc_info=True)
            raise BadCredentials from e
        except github.GithubException as e:
            LOG.error("Repository %s does not exist", repo_name, exc_info=True)
            raise RepositoryDoesNotExist(repo=repo_name) from e
        except Exception as e:
            raise e
        return gh

    def clone(self, branch: str = "main"):
        """Method to clone the repository"""
        if os.path.isdir(self.repo_path):
            LOG.info("Removing existing %s directory", self.repo_name)
            shutil.rmtree(self.repo_path)
        Repo.clone_from(
            self.remote,
            self.repo_path,
            branch=branch,
            multi_options=["--depth=1"],
        )
        LOG.info(
            "Finished cloning repository %s to %s",
            self.repo_name,
            self.repo_path,
        )

    def clone_and_copy(self, copy_from: str, copy_to: str | None = None):
        """Method to clone the repository and copy files from copy_from to copy_to"""
        self.clone()
        path = os.path.join(self.repo_path, copy_to) if copy_to is not None else self.repo_path
        try:
            shutil.copytree(
                copy_from,
                path,
                dirs_exist_ok=True,
                ignore=shutil.ignore_patterns(".*", "_*"),
            )
            LOG.info("Copying complete from %s to %s", copy_from, path)
        except Exception as e:
            LOG.error(
                "Error occured in copying files from %s",
                copy_from,
                exc_info=True,
            )
            raise e

    def switch_branch(self, branch: str):
        """Method to switch to a branch"""
        repo = Repo(self.repo_path)
        repo.git.checkout("-B", branch)
        LOG.info("Switched to %s branch", branch)

    def commit(self, message: str, branch: str = "main"):
        """Method to commit changes to a branch"""
        repo = Repo(self.repo_path)
        self.switch_branch(branch)
        repo.git.add(".")
        try:
            repo.git.commit(m=message)
            LOG.info("Finished git commit")
        except Exception:
            LOG.error("Nothing to commit", exc_info=False)

    def commit_push(self, message: str, branch: str):
        """Method to commit and push changes to a branch"""
        self.commit(message, branch)
        repo = Repo(self.repo_path)
        repo.git.push("--set-upstream", repo.remote().name, branch)

        LOG.info("Pushed to %s", repo.remote().name)

    def create_pr(self, title: str, body: str, head: str, base: str = "main"):
        """Method to create a pull request"""
        try:
            repo_name = self.org + "/" + self.repo_name
            repo = self.github.get_repo(repo_name)
            pr = repo.create_pull(title=title, body=body, head=head, base=base)
            LOG.info("Created a pull request: #%i", pr.number)
            return pr
        except Exception as e:
            LOG.error("Error creating a pull request", exc_info=True)
            raise e

    def merge_pr(self, pr: PullRequest, commit_title: str, commit_message: str):
        """Method to merge a pull request"""
        for _ in range(9):
            try:
                merge_status = pr.merge(commit_message=commit_message, commit_title=commit_title)
                if merge_status.merged:
                    LOG.info("Merge request created successfully")
                    return

                LOG.error(
                    "Could not create merge request - %s",
                    merge_status.message,
                    exc_info=True,
                )
            except Exception:
                LOG.error("Status checks pending...")
                time.sleep(10)

        LOG.error("Pull request #%i could not be merged", pr.number, exc_info=True)
        raise PRMergeError("Pull request could not be merged. Check pending status checks")

    def merge_pr_by_num(self, pr: int, commit_message: str, commit_title: str) -> bool:
        """Method to merge a pull request by number"""
        repo_name = self.org + "/" + self.repo_name
        repo = self.github.get_repo(repo_name)
        LOG.info("Getting pr by number")
        pr = repo.get_pull(pr)

        return self.merge_pr(pr, commit_message=commit_message, commit_title=commit_title)

    def commit_push_delete(self, commit_msg: str, branch_name: str):
        """Method to commit, push and delete the repository"""
        self.commit_push(commit_msg, branch_name)
        shutil.rmtree(self.repo_path)
        LOG.info("Deleted the folder: %s", self.repo_path)
