# pylint: skip-file
""" utility for interacting with kubernetes cluster """
import base64
from kubernetes.client import V1Toleration
from kubernetes import config, client

from pyneers.utils.logger import CustomLogger

# from pyneers.utils.exceptions import JobRunning
from pyneers.models.kubernetes import JobConfig

LOG = CustomLogger.getLogger(__name__)


class KubernetesClient:
    """class for interacting with kubernetes cluster"""

    def __init__(self, cluster_name: str | None = None):
        self.__validate(cluster_name)
        LOG.info("Kubernetes client validated")
        self.core = client.CoreV1Api()
        self.custom = client.CustomObjectsApi()
        self.batch = client.BatchV1Api()

    def __validate(self, name: str | None) -> None:
        """validate kubernetes client"""
        try:
            if name:
                contexts, _ = config.list_kube_config_contexts()
                for context in contexts:
                    if name in context["name"]:
                        LOG.info("Loading kubeconfig for %s", context["name"])
                        config.load_kube_config(context=context["name"])
                        return
            else:
                LOG.info("Loading in-cluster config")
                config.load_incluster_config()
        except Exception as e:
            raise e

    def get_custom_resource(
        self,
        resource: str,
        name: str,
        namespace: str,
        version: str,
        group: str,
    ):
        """get custom resource"""
        LOG.info("Getting custom resource: %s:%s", resource, name)
        try:
            resource = self.custom.get_namespaced_custom_object(
                group=group,
                version=version,
                plural=resource,
                name=name,
                namespace=namespace,
            )
            return resource
        except Exception as e:
            return e

    def get_list_resources(
        self,
        resource: str,
        namespace: str,
        version: str,
        group: str,
    ):
        """get custom resource"""
        LOG.info(f"Resource: {resource}")
        resources = self.custom.list_namespaced_custom_object(
            group=group,
            version=version,
            plural=resource,
            namespace=namespace,
        )
        # LOG.info(f"List of Custom Resources: {resources}")
        return resources

    def get_secret_data(self, _secret: str, namespace: str):
        """get secret data"""
        LOG.info("Reading secret: %s:%s", namespace, _secret)
        data = self.core.read_namespaced_secret(
            _secret,
            namespace,
        ).data
        secrets = {}

        for secret, value in data.items():
            decoded = base64.b64decode(value)
            secrets[secret] = decoded.decode("utf-8")

        return secrets

    def create_job(self, config: JobConfig, namespace: str):
        """create job"""
        LOG.info("Creating new job: %s", config.name)
        containers = [
            client.V1Container(
                name=container.name,
                image=container.image,
                image_pull_policy=container.pull_policy,
                command=container.command,
                env_from=[
                    client.V1EnvFromSource(
                        config_map_ref=client.V1ConfigMapEnvSource(
                            name=container.config_map,
                        )
                    )
                ],
            )
            for container in config.containers
        ]
        tolerations = [
            V1Toleration(
                key="mgmt",
                operator="Equal",
                value="true",
                effect="NoSchedule",
            )
        ]
        template = client.V1PodTemplateSpec(
            spec=client.V1PodSpec(
                restart_policy=config.restart,
                containers=containers,
                service_account_name=config.service_account,
                tolerations=tolerations,
            )
        )
        spec = client.V1JobSpec(template=template, ttl_seconds_after_finished=120)
        job = client.V1Job(
            api_version=config.version,
            kind="Job",
            metadata=client.V1ObjectMeta(name=config.name),
            spec=spec,
        )
        existing = None
        try:
            existing = self.get_job_status(config.name, namespace)
            if existing.active == 1:
                LOG.error("Job %s:%s still running", namespace, config.name)
                # raise JobRunning(name=f"{namespace}:{config.name}")
            if existing.active != 2:
                # only call if existing.active == 0
                self.delete_job(config.name, namespace)
        except Exception as e:
            raise e
        retries = 0
        max_retries = 10
        while retries <= max_retries:
            if self.get_running_pod_count(namespace) < 6:
                try:
                    response = self.batch.create_namespaced_job(body=job, namespace=namespace)
                    LOG.info(f"NUMBER OF RUNNING PODS : {self.get_running_pod_count(namespace)}")
                    return response
                except Exception as e:
                    LOG.warning(f"Request failed: {e}")
                    retries += 1
                    if retries == max_retries:
                        raise e
            else:
                LOG.info(
                    f"NUMBER OF RUNNING PODS EXCEEDED LIMIT: {self.get_running_pod_count(namespace)}"
                )
        return None

    def delete_job(self, name: str, namespace: str) -> client.V1Status:
        """delete job"""
        LOG.info("Deleting job: %s:%s", namespace, name)
        status: client.V1Status = self.batch.delete_namespaced_job(name=name, namespace=namespace)
        return status

    def get_job_status(self, name: str, namespace: str) -> client.V1JobStatus:
        """get job status"""
        LOG.info("Getting job status for - %s:%s", namespace, name)
        try:
            status: client.V1JobStatus = self.batch.read_namespaced_job_status(
                name=name, namespace=namespace
            )
            LOG.info(status.status)
            if status.status.active is None:
                status.status.active = 0
            return status.status
        except client.rest.ApiException as e:
            if e.status == 404:
                LOG.warning("Job %s:%s not found", namespace, name)
                # dont call delete if job not found
                return client.V1JobStatus(active=2)
            else:
                raise e

    def get_running_pod_count(self, namespace: str) -> int:
        """Get the number of running pods in the specified namespace."""
        LOG.info("Getting running pod count for namespace: %s", namespace)
        try:
            pods = self.core.list_namespaced_pod(namespace=namespace)
            running_count = sum(1 for pod in pods.items if pod.status.phase == "Running")
            return running_count
        except client.rest.ApiException as e:
            raise e

    def list_secrets(self, namespace: str):
        """List all secrets in a namespace."""
        try:
            LOG.info("Listing secrets in namespace: %s", namespace)
            secrets_list = self.core.list_namespaced_secret(namespace=namespace)
            secrets_names = [secret.metadata.name for secret in secrets_list.items]
            return secrets_names

        except Exception as e:
            LOG.error("Error listing secrets in namespace %s - %s", namespace, e)
            # You might want to handle the exception differently based on your application's needs.
            return None

    def get_k8s_ingestSecrets(self, customer: str):
        try:
            namespace = customer + "-automation"
            LOG.info(namespace)
            LOG.info("Getting all ingest secrets in namespace: %s", namespace)
            list_secrets = self.list_secrets(namespace=namespace)
            LOG.info(list_secrets)
            ingest_secret = None

            for secret in list_secrets:
                key = secret.split("-", 1)[0]
                if key == "account":
                    ingest_secret = secret
                    break

            LOG.info(ingest_secret)
            result = self.get_secret_data(_secret=ingest_secret, namespace=namespace)
            LOG.info(result)

        except Exception as e:
            LOG.error("Error retreiving ingest secrets %s", e)
            return None

        return result
