"""
Create custom logger to be used in Test
"""
from pathlib import Path
import logging
from abc import ABCMeta
from pyneers import global_logs_dir

ABC = ABCMeta("ABC", (object,), {"__slots__": ()})


class CustomLogger(ABC):
    """
    Custom Logger

    Args:
        ABC (_type_): Builtin Abstract Base Class
    """

    __logger = None

    @staticmethod
    def getLogger(name: str):
        """
        Get Logger
        Args:
            name (str): _description_

        Returns:
            _type_: _description_
        """
        if CustomLogger.__logger is not None:
            return CustomLogger.__logger

        # logs dir to be created under the project dir
        logs_path = Path(global_logs_dir)

        # Create log directory if it does not exist
        if not logs_path.exists():
            logs_path.mkdir(parents=True)

        # Create a root logger and clear any old handlers
        logger = logging.getLogger(name)
        logger.setLevel(logging.DEBUG)

        if logger.handlers:
            for handler in logger.handlers[:]:
                logger.removeHandler(handler)

        # Set formatter for logging
        format_pattern = "%(asctime)s:%(levelname)s:%(filename)s:%(lineno)d:%(message)s"
        formatter = logging.Formatter(format_pattern)

        # Set INFO log handler
        info_handler = logging.FileHandler(logs_path / "info.log")
        info_handler.setLevel(logging.INFO)
        info_handler.setFormatter(formatter)

        # Set DEBUG log handler
        debug_handler = logging.FileHandler(logs_path / "debug.log")
        debug_handler.setLevel(logging.DEBUG)
        debug_handler.setFormatter(formatter)

        # Set ERROR log handler
        error_handler = logging.FileHandler(logs_path / "error.log")
        error_handler.setLevel(logging.ERROR)
        error_handler.setFormatter(formatter)

        # set console log Handler
        console_handler = logging.StreamHandler()
        console_handler.setLevel(logging.INFO)
        console_handler.setFormatter(formatter)

        logger.addHandler(info_handler)
        logger.addHandler(debug_handler)
        logger.addHandler(error_handler)
        logger.addHandler(console_handler)

        # Disable verbosity for other loggers
        urllib3_logger = logging.getLogger("urllib3")
        urllib3_logger.setLevel(logging.ERROR)

        # Disable verbosity for faker logger
        faker_logger = logging.getLogger("faker")
        faker_logger.setLevel(logging.ERROR)

        CustomLogger.__logger = logger
        return CustomLogger.__logger
