"""
Request Utils Module
"""

import asyncio
from typing import Dict, List, Any, Callable, Optional, Mapping
import httpx
import requests
from pyneers.utils.logger import CustomLogger

# Since this library uses asyncio, hence using asyncio logger
LOG = CustomLogger.getLogger("asyncio")

# Httpx Event Hooks - https://www.python-httpx.org/advanced/#request-instances


def log_request(request: httpx.Request) -> None:
    """
    Event hook for httpx for logging request
    Args:
        request: Httpx request object.
    Returns:
        None.
    """
    LOG.debug(f"Request event hook: {request.method} {request.url} - Waiting for response")


def log_response(response: httpx.Response) -> None:
    """
    Event hook for httpx for logging response
    Args:
        response: Httpx response object.
    Returns:
        return: None.
    """
    request = response.request

    LOG.debug(
        f"Response event hook: {request.method} {request.url} - Status {response.status_code}"
    )


def merge_url_path(url: str = "", path: str = "") -> str:
    """
    Merge a base URL and a path, ensuring a single slash between them.

    Args:
        url (str): The base URL.
        path (str): The path to be appended.
    Returns:
        str: The merged URL with a single slash between the base URL and the path.
    """
    if path == "":
        final_url = f"{url.strip().rstrip('/')}{path.strip().lstrip('/')}"
    else:
        final_url = f"{url.strip().rstrip('/')}/{path.strip().lstrip('/')}"
    return final_url


def get_async_semaphore(max_concurrent_connections: int) -> asyncio.Semaphore:
    """
    Get Semaphore object
    Args:
        max_concurrent_connections: Max concurrent connections.
    Returns:
        return: Semaphore object.
    """
    return asyncio.Semaphore(max_concurrent_connections)


def prepare_async_request(method: str, url: str, *args: Any, **kwargs) -> httpx.Request:
    """
    This function helps in preparing request. It works for both sync and async client.
    Via Kwargs, you can send all the keyword arguments supported by httpx client.
    Reference: https://www.python-httpx.org/advanced/#request-instances
    Sample usage:
        request = prepare_async_request("GET", "https://www.google.com")
        # Sync client         = httpx.Client(**kwargs)
        # Async client         = httpx.AsyncClient(**kwargs)
        response = client.send(request)
        print(response, response.num_bytes_downloaded )
    Args:
        method (str): Method to call the endpoint. Example - GET, POST, PUT, PATH, DELETE, etc.
        url (str): Base url for the endpoint.
        args: Additional arguments for the request.
        kwargs: Additional keywords arguements.
    Returns:
        Httpx.Request object: Request object.
    """
    return httpx.Request(method, url, *args, **kwargs)


async def hit_async_request(
    method: str,
    url: str,
    *args: Any,
    path: Optional[str] = "",
    params: Dict[str, Any] = None,
    headers: Dict[str, Any] = None,
    payload: Dict[str, Any] = None,
    client: Optional[httpx.AsyncClient] = None,
    event_hooks: Optional[Mapping[str, List[Callable[..., Any]]]] = None,
    close_connection: Optional[bool] = False,
    timeout: Optional[int] = 10,
    **kwargs: Any,
) -> httpx.Response:
    """
    ** Make sure to await this function/coroutine **
    Base function to handle asynchronus api request.
    Args:
        method (str): Method to call the endpoint. Example - GET, POST, PUT, PATH, DELETE, etc.
        url (str): The URL for the API call.
        args: Additional arguments for the request.
        path (str, Optional): Additional path to append to the base URL. Defaults to "".
        params (dict,Optional): Query params to be added to the url.
        headers (dict,Optional): Header to be sent with the request.
        payload (dict,Optional): Payload to be sent with the request.
        client (httpx.AsyncClient,Optional): Existing async client.
        event_hooks (Optional): Event hooks that are to be injected to httpx library.
        close_connection (bool,Optional): whether to close the connection at exit.
        timeout(float): Duration to wait for response.
        kwargs: Additional keyword arguments for request.

    Returns:
        httpx.Response: Response for the request.
    """
    if path is None:
        path = ""
    if headers is None:
        headers = {}
    if params is None:
        params = {}
    if client is None:
        client = httpx.AsyncClient(*args, event_hooks=event_hooks, timeout=timeout, **kwargs)
        # Since connection is being created in this function, hence setting it to True
        close_connection = True
    url = merge_url_path(url, path)
    if payload is None:
        response = await client.request(
            *args, method=method, url=url, headers=headers, params=params, **kwargs
        )
    else:
        response = await client.request(
            *args, method=method, url=url, headers=headers, params=params, json=payload, **kwargs
        )
    if close_connection:
        await client.aclose()
    return response


async def hit_async_requests(
    requests: List[httpx.Request],
    *args: Any,
    max_concurrent_connections: Optional[int] = 10,
    event_hooks: Optional[Mapping[str, List[Callable[..., Any]]]] = None,
    client: Optional[httpx.AsyncClient] = None,
    semaphore: Optional[asyncio.Semaphore] = None,
    close_connection: Optional[bool] = False,
    timeout: Optional[int] = 10,
    **kwargs: Any,
) -> List[httpx.Response]:
    """
    This function can be used to hit a list of pre-configured urls with the same headers & payload using httpx module.

    Args:
        requests ([httpx.Request]): List of prepared requests of type httpx.Request.
        args: Additional arguments for the request.
        max_concurrent_connections (int): Maximum number of concurrent connections to be made while async requests.
        event_hooks (Optional): Any event hooks for httpx request.
        client (httpx.AsyncClient,Optional): For using existing async client.
        semaphore (asyncio.Semaphore,Optional): For using existing semaphore.
        close_connection (bool,Optional): whether to close the connection at exit.Default to False.
        timeout(int): Duration to wait for response.Default to 10.
        kwargs: Additional keyword arguments for request.

    Returns:
        List[httpx.Response]: List of httpx.response for the requests.
    """
    try:
        if max_concurrent_connections is None:
            max_concurrent_connections = 10
        # Pass existing semaphore to control concurrent connections
        if semaphore is None:
            semaphore = get_async_semaphore(max_concurrent_connections)
        if client is None:
            client = httpx.AsyncClient(*args, event_hooks=event_hooks, timeout=timeout, **kwargs)
            # Since connection is being created in this function, hence setting it to True
            close_connection = True
        # This way is being used to give an option to re-use the connection
        # async with httpx.AsyncClient(*args, event_hooks=event_hooks, **kwargs) as client:
        responses: List[Any] = []
        # Using TaskGroup to create async tasks and are awaited.
        # Refer: https://docs.python.org/3/library/asyncio-task.html#asyncio.TaskGroup
        async with semaphore:
            async with asyncio.TaskGroup() as tg:
                for request in requests:
                    response = await tg.create_task(client.send(request=request))
                    responses.append(response)
        # Explicitly close the connection
        if close_connection:
            await client.aclose()
        for response in responses:
            LOG.debug(f"RESPONSE CONTENT:{response.content}")
    except Exception as ex:
        LOG.error(f"Error calling requests:{ex}")
        return ex

    return responses


class AsyncRestClient:

    """
    Utility class for asynchronus Rest APIs support using the httpx module.
    """

    def __init__(
        self,
        base_url: str,
        *args: Any,
        max_concurrent_connections: Optional[int] = 10,
        timeout: Optional[float] = 10,
        http2: Optional[bool] = False,
        **kwargs: Any,
    ):
        """
        Initialize the RestClient with the base URL of the REST API.

        Args:
            base_url (str): The URL for the API call.
            args: Additional arguments for the requests.
            max_concurrent_connections(int): To set maximum no of api calls at instance.
            timeout(float): Duration to wait for response.
            http2 (bool): To enable http2 in httpx module.
            kwargs: Additional keyword arguments for requests.
        """

        try:
            self.base_url = base_url

            if max_concurrent_connections is None or not isinstance(
                max_concurrent_connections, int
            ):
                max_concurrent_connections = 10

            self.http2 = bool(http2)

            self.timeout = timeout

            self.max_concurrent_connections = max_concurrent_connections

            self.semaphore = get_async_semaphore(max_concurrent_connections)

            self.client = httpx.AsyncClient(*args, timeout=self.timeout, **kwargs)

        except Exception as e:
            LOG.error(f"Error initialising AsyncRestClient:{e}")

    async def __aenter__(self):
        """
        __     This method is called at the start of with Class.
        """
        return self

    async def __aexit__(self, *args):
        """
        __     This method is called at the end of with Class.
        """
        await self.close_async_connection()

    def get_async_client(self):
        """
        __     This method returns httpx client of the Class.
        """
        return self.client

    async def close_async_connection(self) -> None:
        """
        __     This method is for closing async connection to given url.
        """
        LOG.info(f"Closing async connection to {self.base_url}")

        await self.client.aclose()

    async def get(
        self,
        path: Optional[str] = "",
        **kwargs: Any,
    ) -> httpx.Response:
        """

        Send a GET request.

            Args:
                path (str, Optional): Additional path to append to the base URL. Defaults to "".
                kwargs: Additional keyword arguments for the request.

            Returns:
                httpx.Response: Response for the GET request.
        """
        response = await hit_async_request(
            method="GET",
            client=self.client,
            url=self.base_url,
            path=path,
            **kwargs,
        )

        try:
            LOG.debug(f"RESPONSE CONTENT:{response.content}")
            return response
        except Exception as ex:
            LOG.error(f"Error calling GET request:{ex}")
            return ex

    async def post(
        self,
        path: Optional[str] = "",
        **kwargs: Any,
    ) -> httpx.Response:
        """

        Send a POST request.

        Args:
            path (str, Optional): Additional path to append to the base URL. Defaults to "".
            kwargs: Additional keyword arguments for the request.

        Returns:
            httpx.Response: Response for the POST request.
        """

        response = await hit_async_request(
            method="POST",
            client=self.client,
            path=path,
            url=self.base_url,
            **kwargs,
        )

        try:
            LOG.debug(f"RESPONSE CONTENT:{response.content}")
            return response
        except Exception as ex:
            LOG.error(f"Error calling POST request:{ex}")
            return ex

    async def put(
        self,
        path: Optional[str] = "",
        **kwargs: Any,
    ) -> httpx.Response:
        """

        Send a PUT request.

        Args:
            path (str, Optional): Additional path to append to the base URL. Defaults to "".
            kwargs: Additional keyword arguments for the request.

        Returns:
            httpx.Response: Response for the PUT request.
        """

        response = await hit_async_request(
            method="PUT",
            client=self.client,
            url=self.base_url,
            path=path,
            **kwargs,
        )
        try:
            LOG.debug(f"RESPONSE CONTENT:{response.content}")
            return response
        except Exception as ex:
            LOG.error(f"Error calling PUT request:{ex}")
            return ex

    async def patch(
        self,
        path: Optional[str] = "",
        **kwargs: Any,
    ) -> httpx.Response:
        """

        Send a PATCH request.

        Args:
            path (str, Optional): Additional path to append to the base URL. Defaults to "".
            kwargs: Additional keyword arguments for the request.

        Returns:
            httpx.Response: Response for the PATCH request.
        """
        response = await hit_async_request(
            method="PATCH",
            client=self.client,
            url=self.base_url,
            path=path,
            **kwargs,
        )

        try:
            LOG.debug(f"RESPONSE CONTENT:{response.content}")
            return response
        except Exception as ex:
            LOG.error(f"Error calling PATCH request:{ex}")
            return ex

    async def delete(
        self,
        path: Optional[str] = "",
        **kwargs: Any,
    ) -> httpx.Response:
        """

        Send a DELETE request.

        Args:
            path (str, Optional): Additional path to append to the base URL. Defaults to "".
            kwargs: Additional keyword arguments for the request.
        Returns:
            httpx.Response: Response for the DELETE request.

        """
        response = await hit_async_request(
            method="DELETE",
            client=self.client,
            url=self.base_url,
            path=path,
            **kwargs,
        )
        try:
            LOG.debug(f"RESPONSE CONTENT:{response.content}")
            return response
        except Exception as ex:
            LOG.error(f"Error calling DELETE request:{ex}")
            return ex


class AsyncGraphQLClient:
    """
    Utility class for calling asynchronus GraphQL queries using httpx module.
    """

    def __init__(
        self,
        endpoint_url: str,
        max_concurrent_connections: Optional[int] = 10,
        timeout: Optional[float] = 10,
        http2: Optional[bool] = False,
        **kwargs: Any,
    ):
        """
        Initialize the GraphQLClient with the end point url given by user.

        Args:
            endpoint_url(str): The endpoint URL for the GraphQL query.
            max_concurrent_connections(int,Optional): To set maximum no of api calls at instance.
            timeout(float): Duration to wait for response.
            http2 (bool,Optional): To enable http2 in httpx module.
            kwargs: Additional keyword arguments for requests.

        """

        try:
            self.endpoint_url = endpoint_url
            if max_concurrent_connections is None or not isinstance(
                max_concurrent_connections, int
            ):
                max_concurrent_connections = 10
            self.http2 = bool(http2)
            self.timeout = timeout
            self.max_concurrent_connections = max_concurrent_connections
            self.semaphore = get_async_semaphore(max_concurrent_connections)
            self.client = httpx.AsyncClient(timeout=self.timeout, **kwargs)

        except Exception as e:
            LOG.error(f"Error initialising AsyncGraphQLClient:{e}")

    async def __aenter__(self):
        """
        __     This method is called at the start of with Class.
        """
        return self

    async def __aexit__(self, *args, **kwargs):
        """
        __     This method is called at the end of with Class.
        """
        await self.close_async_connection()

    def get_async_client(self):
        """
        __     This method returns httpx client of the Class.
        """
        return self.client

    async def close_async_connection(self) -> None:
        """
        __     This method is for closing async connection to given url.
        """

        LOG.info(f"Closing async connection to {self.endpoint_url}")
        await self.client.aclose()

    async def execute_query(
        self,
        path: Optional[str] = "",
        query: str = "",
        variables: Dict[str, Any] = None,
        **kwargs: Any,
    ) -> httpx.Response:
        """

        Send a async GraphQL Query using httpx module.

        Args:
            path (str, Optional): Additional path to append to the base URL. Defaults to "".
            query (str): The query string.
            variables (Dict,Optional): Variables for the GraphQL query.
            kwargs: Additional keyword arguments for the request.

        Returns:
            httpx.Response: Response for the GraphQL query.
        """
        payload = {"query": query, "variables": variables}
        response = await hit_async_request(
            method="POST",
            path=path,
            client=self.client,
            url=self.endpoint_url,
            payload=payload,
            **kwargs,
        )
        try:
            LOG.debug(f"RESPONSE CONTENT:{response.content}")
            return response
        except Exception as ex:
            LOG.error(f"Error calling GraphQL request:{ex}")
            return ex


def multiple_url_requests(
    requests: List[httpx.Request],
    *args: Any,
    event_hooks: Optional[Dict[str, List[Callable[..., Any]]]] = None,
    timeout: Optional[int] = 10,
    **kwargs: Any,
) -> List[httpx.Response]:
    """
    This function can be used to hit a list of pre-configured URLs with the same headers & payload using httpx module.

    Args:
        requests (List[httpx.Request]): List of prepared requests of type httpx.Request.
        args: Additional arguments for the request.
        event_hooks (Optional): Any event hooks for httpx request.
        timeout(int): Duration to wait for response. Default to 10.
        kwargs: Additional keyword arguments for the request.

    Returns:
        List[httpx.Response]: List of httpx.response for the requests.
    """
    try:
        responses: List[httpx.Response] = []

        with httpx.Client(*args, timeout=timeout, event_hooks=event_hooks, **kwargs) as client:
            for request in requests:
                response = client.send(request=request)
                responses.append(response)
        for response in responses:
            LOG.debug(f"RESPONSE CONTENT:{response.content}")
    except Exception as ex:
        LOG.error(f"Error calling requests:{ex}")
        return ex

    return responses


class RestClient:
    """
    Utility class for making REST API calls using the requests module.
    """

    def __init__(
        self,
        base_url: str,
        *args: Any,
        headers: Dict[str, str] = None,
        timeout: Optional[float] = 10,
        create_session: Optional[bool] = False,
        **kwargs: Any,
    ):
        """
        Initialize the RestClient.

        Args:
            base_url (str): The base URL of the REST API.
            args: Additonal arguments for requests.
            headers (dict, Optional): Headers for the requests. Defaults to None.
            timeout (float, Optional): Timeout duration for the requests. Defaults to 10.
            create_session (bool, Optional): To enable requests.session. Defaults to False.
            kwargs: Additional keyword arguments for requests.

        """
        self.timeout = timeout
        self.headers = headers
        self.base_url = base_url
        self.additional_args = args
        self.additional_key_args = kwargs
        self.create_session = create_session

        if self.create_session:
            self.session = requests.Session()
            self.session.prefix = base_url
            self.session.headers = self.headers
            self.session.timeout = self.timeout
            for key, value in self.additional_key_args.items():
                setattr(self.session, key, value)

    def __enter__(self):
        """
        Enter the context manager.
        """
        return self

    def __exit__(self, exc_type, exc_value, traceback):
        """
        Exit the context manager.
        """
        if self.create_session:
            self.session.close()

        LOG.info("Exiting RestClient Class")

    def close_session(self):
        """
        Function for closing requests.session.
        """
        if self.create_session:
            self.session.close()

    def send_request(
        self,
        method: str,
        url: str,
        *args: Any,
        data: Optional[Dict[str, Any]] = None,
        headers: Dict[str, str] = None,
        **kwargs: Any,
    ) -> requests.Response:
        """
        Send a prepared request and handle exceptions.

        Args:
            method (str): HTTP method (GET, POST, PUT, PATCH, DELETE).
            url (str): The URL for the API call.
            args: Additional arguments for the request.
            data (dict, Optional): Data for the request body. Defaults to None.
            headers (dict, Optional): Additional headers for the request. Defaults to None.
            kwargs: Additional keyword arguments for requests.

        Returns:
            requests.Response: Response object.

        """
        try:
            if self.create_session:
                response = self.session.request(
                    method,
                    *args,
                    url=url,
                    json=data,
                    headers=headers,
                    **kwargs,
                )
            else:
                headers = headers or self.headers
                response = requests.request(
                    method,
                    *self.additional_args,
                    *args,
                    url=url,
                    json=data,
                    headers=headers,
                    timeout=self.timeout,
                    **self.additional_key_args,
                    **kwargs,
                )
            LOG.debug(f"RESPONSE CONTENT:{response.content}")
            return response
        except requests.RequestException as ex:
            LOG.error(f"Request Exception: {ex}")
            raise ex

    def get(
        self,
        path: Optional[str] = "",
        headers: Dict[str, str] = None,
        **kwargs: Any,
    ) -> requests.Response:
        """
        Send a GET request.

        Args:
            path (str, Optional): Additional path to append to the base URL. Defaults to "".
            headers (dict, Optional): Additional headers for the request. Defaults to None.
            kwargs: Additional keyword arguments for the request.

        Returns:
            requests.Response: Response for the GET request.

        """
        return self.send_request(
            method="GET", url=merge_url_path(self.base_url, path), headers=headers, **kwargs
        )

    def post(
        self,
        path: Optional[str] = "",
        json: Optional[Dict[str, Any]] = None,
        headers: Dict[str, str] = None,
        **kwargs: Any,
    ) -> requests.Response:
        """
        Send a POST request.

        Args:
            path (str, Optional): Additional path to append to the base URL. Defaults to "".
            json (dict, Optional): Data for the request body. Defaults to None.
            headers (dict, Optional): Additional headers for the request. Defaults to None.
            kwargs: Additional keyword arguments for the request.

        Returns:
            requests.Response: Response for the POST request.

        """
        return self.send_request(
            method="POST",
            url=merge_url_path(self.base_url, path),
            data=json,
            headers=headers,
            **kwargs,
        )

    def put(
        self,
        path: Optional[str] = "",
        json: Optional[Dict[str, Any]] = None,
        headers: Dict[str, str] = None,
        **kwargs: Any,
    ) -> requests.Response:
        """
        Send a PUT request.

        Args:
            path (str, Optional): Additional path to append to the base URL. Defaults to "".
            json (dict, Optional): Data for the request body. Defaults to None.
            headers (dict, Optional): Additional headers for the request. Defaults to None.
            kwargs: Additional keyword arguments for the request.

        Returns:
            requests.Response: Response for the PUT request.

        """
        return self.send_request(
            method="PUT",
            url=merge_url_path(self.base_url, path),
            data=json,
            headers=headers,
            **kwargs,
        )

    def patch(
        self,
        path: Optional[str] = "",
        json: Optional[Dict[str, Any]] = None,
        headers: Dict[str, str] = None,
        **kwargs: Any,
    ) -> requests.Response:
        """
        Send a PATCH request.

        Args:
            path (str, Optional): Additional path to append to the base URL. Defaults to "".
            json (dict, Optional): JSON data for the request body. Defaults to None.
            headers (dict, Optional): Additional headers for the request. Defaults to None.
            kwargs: Additional keyword arguments for the request.

        Returns:
            requests.Response: Response for the PATCH request.

        """
        return self.send_request(
            method="PATCH",
            url=merge_url_path(self.base_url, path),
            data=json,
            headers=headers,
            **kwargs,
        )

    def delete(
        self,
        path: Optional[str] = "",
        headers: Dict[str, str] = None,
        **kwargs: Any,
    ) -> requests.Response:
        """
        Send a DELETE request.

        Args:
            path (str, Optional): Additional path to append to the base URL. Defaults to "".
            headers (dict, Optional): Additional headers for the request. Defaults to None.
            kwargs: Additional keyword arguments for the request.

        Returns:
            requests.Response: Response for the DELETE request.

        """
        return self.send_request(
            method="DELETE", url=merge_url_path(self.base_url, path), headers=headers, **kwargs
        )


class GraphQLClient:
    """
    Utility class for making GraphQL API calls using the requests module.
    """

    def __init__(
        self,
        endpoint_url: str,
        headers: Dict[str, str] = None,
        timeout: Optional[float] = 10,
        create_session: Optional[bool] = False,
        **kwargs: Any,
    ):
        """
        Initialize the GraphQLClient.

        Args:
            endpoint_url (str): The endpoint URL for the GraphQL API.
            headers (dict, Optional): Headers for the requests. Defaults to None.
            timeout (float, Optional): Timeout duration for the requests. Defaults to 10.
            create_session (bool, Optional): To enable requests.session. Defaults to False.
            kwargs: Additional keyword arguments for requests.

        """
        self.timeout = timeout
        self.headers = headers
        self.endpoint_url = endpoint_url
        self.additional_key_args = kwargs
        self.create_session = create_session

        if self.create_session:
            self.session = requests.Session()
            self.session.headers = self.headers
            self.session.timeout = self.timeout
            for key, value in self.additional_key_args.items():
                setattr(self.session, key, value)

    def __enter__(self):
        """
        Enter the context manager.
        """
        return self

    def __exit__(self, exc_type, exc_value, traceback):
        """
        Exit the context manager.
        """

        if self.create_session:
            self.session.close()

        LOG.info("Exiting GraphQLClient Class")

    def close_session(self):
        """
        Function for closing requests.session.
        """
        if self.create_session:
            self.session.close()

    def send_request(
        self,
        url: str,
        *args: Any,
        data: Optional[Dict[str, Any]] = None,
        headers: Dict[str, str] = None,
        **kwargs: Any,
    ) -> requests.Response:
        """
        Send a prepared request and handle exceptions.

        Args:
            url (str): The URL for the API call.
            args: Additional arguments for the request.
            data (dict, Optional): Data for the request body. Defaults to None.
            headers (dict, Optional): Additional headers for the request. Defaults to None.
            kwargs: Additional keyword arguments for requests.

        Returns:
            requests.Response: Response object.

        """
        try:
            headers = headers or self.headers
            if self.create_session:
                response = self.session.request(
                    *args,
                    method="POST",
                    url=url,
                    json=data,
                    headers=headers,
                    timeout=self.timeout,
                    **self.additional_key_args,
                    **kwargs,
                )
            else:
                response = requests.request(
                    *args,
                    method="POST",
                    url=url,
                    json=data,
                    headers=headers,
                    timeout=self.timeout,
                    **self.additional_key_args,
                    **kwargs,
                )
            LOG.debug(f"RESPONSE CONTENT:{response.content}")
            return response
        except requests.RequestException as ex:
            LOG.error(f"Request Exception: {ex}")
            raise ex

    def execute_query(
        self,
        query: str,
        variables: Optional[Dict[str, Any]] = None,
        headers: Dict[str, str] = None,
        **kwargs: Any,
    ) -> requests.Response:
        """
        Send a GraphQL query using the requests module.

        Args:
            query (str): The GraphQL query string.
            variables (dict, Optional): Variables for the GraphQL query. Defaults to None.
            headers (dict, Optional): Additional headers for the request. Defaults to None.
            kwargs: Additional keyword arguments for requests.

        Returns:
            requests.Response: The response object.

        """
        data = {"query": query, "variables": variables}
        return self.send_request(url=self.endpoint_url, data=data, headers=headers, **kwargs)
