# flake8: noqa
"""
    Module containing Slack Message Handler Methods
"""

from typing import List, Union
from datetime import datetime
from pydantic import BaseModel
from pyneers.utils.logger import CustomLogger
from pyneers.utils.request import RestClient

LOG = CustomLogger.getLogger(__name__)


class SlackField(BaseModel):
    """
    Represents a field in a Slack message attachment.

    Attributes:
        title (str): The title of the field.
        value (str): The value of the field.
        short (bool): A boolean indicating whether the field should be displayed
            in a short format in the Slack message attachment.
    """

    title: str
    value: Union[str, dict, int, float, bool]
    short: bool


class SlackAttachment(BaseModel):
    """
    Represents a Slack message attachment.

    Attributes:
        color (str): The color of the attachment, typically in hexadecimal format.
        pretext (str): The pretext or introductory text for the attachment.
        fields (List[SlackField]): A list of SlackField objects representing the
            fields to be included in the attachment.
    """

    color: str
    pretext: str
    fields: List[SlackField]


class SlackMessagePayload(BaseModel):
    """
    Represents a Slack message payload.

    Attributes:
        attachments (List[SlackAttachment]): A list of SlackAttachment objects
            representing the attachments to be included in the message.
    """

    attachments: List[SlackAttachment]


def send_test_report_to_slack(slack_webhook_url, report_data):
    """
    Post a message to a Slack channel with a test report.

    Args:
        slack_webhook_url (str): The URL of the Slack webhook to which the message
            should be posted.
        report_data (dict): The test report data to be included in the message.

    Raises:
        Exception: If there is an error while posting the message to Slack.

    """

    headers = {"Content-Type": "application/json"}
    try:
        client = RestClient(slack_webhook_url)
        client.post(json=report_data, headers=headers)
        LOG.info("Test report sent to Slack successfully!")
    except Exception as e:
        LOG.error(f"Failed to send test report to Slack: {e}")


def create_dynamic_payload(report_data, report_path=None, **kwargs):
    """
    Create a dynamic Slack message payload based on the provided test report data.

    Args:
        report_data (dict): The test report data used to generate the payload.
        report_path (str, optional): The path of the test report. Defaults to None.
        **kwargs: Additional key-value pairs to be included as Slack fields.

    Returns:
        str: A JSON-formatted string representing the Slack message payload.

    """
    if not report_data:
        LOG.error("Failed to generate report payload")
        return None

    test_summary = report_data.get("summary", {})
    total = test_summary.get("total", 0)
    passed = test_summary.get("passed", 0)
    skipped = test_summary.get("skipped", 0)
    collected = test_summary.get("collected", 0)
    failed = test_summary.get("failed", 0)
    error = test_summary.get("error", 0)
    deselected = test_summary.get("deselected", 0)
    created_date = report_data.get("created", 0)
    duration = report_data.get("duration", 0)

    fields = [
        SlackField(
            title="Created",
            value=datetime.fromtimestamp(created_date).strftime("%Y-%m-%d %H:%M:%S"),
            short=False,
        ),
        SlackField(
            title="Summary",
            value=f"Total: {total}\n"
            f"Passed: {passed}\n"
            f"Failed: {failed}\n"
            f"Error: {error}\n"
            f"Skipped: {skipped}\n"
            f"Collected: {collected}\n"
            f"Deselected: {deselected}\n",
            short=False,
        ),
        SlackField(
            title="Status",
            value="All tests have passed"
            if failed == 0 == error
            else f"{failed} tests have failed",
            short=False,
        ),
        SlackField(
            title="Duration",
            value=f"{duration:.2f} seconds",
            short=False,
        ),
    ]

    # add additional fields if any passed in kwargs
    for key, value in kwargs.items():
        fields.append(
            SlackField(
                title=key.capitalize(),
                value=str(value),
                short=False,
            )
        )

    if report_path:
        fields.append(
            SlackField(
                title="Report Path",
                value=report_path,
                short=False,
            )
        )

    all_tests_passed = failed == 0 == error
    green = "#36a64f"
    red = "#FF0000"
    status_color = green if all_tests_passed else red

    attachment = SlackAttachment(
        color=status_color, pretext="*Test Automation Report Summary*", fields=fields
    )

    payload = SlackMessagePayload(attachments=[attachment])
    final_payload = payload.model_dump(mode="json")
    return final_payload
