""" Handler for text files handling """
from functools import lru_cache
from pyneers.utils.base_handler import BaseHandler
from pyneers.utils.logger import CustomLogger
from pyneers.utils.exceptions import FileReadError, FileWriteError

LOG = CustomLogger.getLogger(__name__)


# pylint: disable=duplicate-code
class TextHandler(BaseHandler):
    """Class responsible for handling of text files"""

    @lru_cache(maxsize=128)
    def cache_read(self, encoding="utf-8") -> str:
        """Method to read text files using cache

        Args:
            encoding (str, optional): Type of encoding to be used. Defaults to "utf-8".

        Raises:
            FilePathError: Raises FilePathError if file path is incorrect and does not exist
            FileSizeError: Raises FileSizeError if file size is more than 500 MB
            FileTypeError: Raises FileTypeError if file type is incorrect
            FileReadError: Raises FileReadError if it is unable to open or read the file

        Returns:
            File Data (str): Returns file data in string format in case of successful read
        """
        self.validate_file()
        try:
            with open(self.file_path, "r", encoding=encoding) as f:
                return f.read()
        except Exception as e:
            LOG.error(e)
            raise FileReadError() from e

    def read(self, encoding="utf-8") -> str:
        """Method to read text files

        Args:
            encoding (str, optional): Type of encoding to be used. Defaults to "utf-8".

        Raises:
            FilePathError: Raises FilePathError if file path is incorrect and does not exist
            FileSizeError: Raises FileSizeError if file size is more than 500 MB
            FileTypeError: Raises FileTypeError if file type is incorrect
            FileReadError: Raises FileReadError if it is unable to open or read the file

        Returns:
            File Data (str): Returns file data in string format in case of successful read
        """
        self.validate_file()
        try:
            with open(self.file_path, "r", encoding=encoding) as f:
                return f.read()
        except Exception as e:
            LOG.error(e)
            raise FileReadError() from e

    def write(self, data: str, mode="w", encoding="utf-8") -> None:
        """Method to write to text files

        Args:
            data (str): Data to be written to the file
            mode (str, optional): Mode for writing to the file. Uses "w" mode to write to the file
            and "a" mode to append to the file. Defaults to "w".
            encoding (str, optional): Type of encoding to be used. Defaults to "utf-8".

        Raises:
            FilePathError: Raises FilePathError if file path is incorrect and does not exist
            FileSizeError: Raises FileSizeError if file size is more than 500 MB
            FileTypeError: Raises FileTypeError if file type is incorrect
            FileWriteError: Raises FileWriteError if it is unable to open or write to a file

        Returns:
            None: Returns nothing in case of a successful write
        """
        self.validate_file()
        try:
            with open(self.file_path, mode, encoding=encoding) as f:
                f.write(data)

        except Exception as e:
            LOG.error(e)
            raise FileWriteError() from e
