"""Handler for YAML file handling"""
from functools import lru_cache
import yaml
from ruamel.yaml import YAML
from pyneers.utils.base_handler import BaseHandler
from pyneers.utils.logger import CustomLogger
from pyneers.utils.exceptions import FileLoadError, FileReadError, FileWriteError

LOG = CustomLogger.getLogger(__name__)


# pylint: disable=duplicate-code
class YAMLHandler(BaseHandler):
    """Class responsible for having YAML handler"""

    @lru_cache(maxsize=128)
    def cache_read(self, encoding="utf-8", parser="pyyaml") -> str:
        """Method to read yaml files using cache

        Args:
            encoding (str, optional): Type of encoding to be used. Defaults to "utf-8".
             parser (str, optional): Parser to be used. Supports pyyaml and ruamel parser. Defaults to "pyyaml".
        Raises:
            FilePathError: Raises FilePathError if file path is incorrect and does not exist
            FileSizeError: Raises FileSizeError if file size is more than 500 MB
            FileTypeError: Raises FileTypeError if file type is incorrect
            FileReadError: Raises FileReadError if it is unable to open or read the file

        Returns:
            File Data (str): Returns file data in string format in case of successfull read
        """
        self.validate_file()
        try:
            with open(self.file_path, "r", encoding=encoding) as f:
                if parser == "pyyaml":
                    data = yaml.safe_load(f)
                    return data

                yaml_ = YAML()
                data = yaml_.load(f)
                return data
        except Exception as e:
            LOG.error(e)
            raise FileReadError() from e

    def read(self, encoding="utf-8", parser="pyyaml") -> str:
        """Method to read yaml files

        Args:
            encoding (str, optional): Type of encoding to be used. Defaults to "utf-8".
            parser (str, optional): Parser to be used. Supports pyyaml and ruamel parser. Defaults to "pyyaml".

        Raises:
            FilePathError: Raises FilePathError if file path is incorrect and does not exist
            FileSizeError: Raises FileSizeError if file size is more than 500 MB
            FileTypeError: Raises FileTypeError if file type is incorrect
            FileReadError: Raises FileReadError if it is unable to open or read the file

        Returns:
            File Data (str): Returns file data in string format in case of successfull read
        """
        self.validate_file()
        try:
            with open(self.file_path, "r", encoding=encoding) as f:
                if parser == "pyyaml":
                    data = yaml.safe_load(f)
                    return data

                yaml_ = YAML()
                data = yaml_.load(f)
                return data
        except Exception as e:
            LOG.error(e)
            raise FileReadError() from e

    def write(self, data: str, encoding="utf-8", parser="pyyaml") -> None:
        """Method to write to yaml files

        Args:
            data (str): Data to be written to the file
            encoding (str, optional): Type of encoding to be used. Defaults to "utf-8".
            parser (str, optional): Parser to be used. Supports pyyaml and ruamel parser. Defaults to "pyyaml".

        Raises:
            FilePathError: Raises FilePathError if file path is incorrect and does not exist
            FileSizeError: Raises FileSizeError if file size is more than 500 MB
            FileTypeError: Raises FileTypeError if file type is incorrect
            FileWriteError: Raises FileWriteError if it is unable to open or write to a file

        Returns:
            None: Returns nothing in case of a successful write
        """
        self.validate_file()
        try:
            with open(self.file_path, "w", encoding=encoding) as f:
                if parser == "pyyaml":
                    yaml.dump(data, f)
                else:
                    # use ruaeml yaml parser
                    yaml_ = YAML()
                    yaml_.dump(data, f)

        except Exception as e:
            LOG.error(e)
            raise FileWriteError() from e

    def load_yaml(self, data: object, parser: str = "pyyaml"):
        """Method to load yaml objects

        Args:
            data (str): Data to be written to the file
            parser (str, optional): Parser to be used. Supports pyyaml and ruamel parser. Defaults to "pyyaml".

        Raises:
            FilePathError: Raises FilePathError if file path is incorrect and does not exist
            FileSizeError: Raises FileSizeError if file size is more than 500 MB
            FileTypeError: Raises FileTypeError if file type is incorrect
            FileWriteError: Raises FileWriteError if it is unable to open or write to a file

        Returns:
            loaded yaml: Returns loaded yaml object
        """
        try:
            if parser == "pyyaml":
                loaded = yaml.safe_load(data)
            elif parser == "ruamel":
                yaml_ = YAML()
                loaded = yaml_.load(data)
            return loaded
        except Exception as e:
            LOG.error(e)
            raise FileLoadError() from e
